from collections import defaultdict

import datasets
from datasets import DatasetDict

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

# Language codes for Common Voice 17
_EVAL_LANGS_CV17 = {
    "ar": ["ara-Arab"],
    "ast": ["ast-Latn"],
    "be": ["bel-Cyrl"],
    "bg": ["bul-Cyrl"],
    "bn": ["ben-Beng"],
    "br": ["bre-Latn"],
    "cs": ["ces-Latn"],
    "cy": ["cym-Latn"],
    "da": ["dan-Latn"],
    "de": ["deu-Latn"],
    "el": ["ell-Grek"],
    "en": ["eng-Latn"],
    "es": ["spa-Latn"],
    "et": ["est-Latn"],
    "fa": ["fas-Arab"],
    "fi": ["fin-Latn"],
    "fr": ["fra-Latn"],
    "frold": ["fro-Latn"],
    "gl": ["glg-Latn"],
    "ha": ["hau-Latn"],
    "hi": ["hin-Deva"],
    "hu": ["hun-Latn"],
    "it": ["ita-Latn"],
    "ja": ["jpn-Jpan"],
    "ka": ["kat-Geor"],
    "ko": ["kor-Hang"],
    "lt": ["lit-Latn"],
    "lv": ["lav-Latn"],
    "mk": ["mkd-Cyrl"],
    "ml": ["mal-Mlym"],
    "mn": ["mon-Cyrl"],
    "mr": ["mar-Deva"],
    "nl": ["nld-Latn"],
    "oc": ["oci-Latn"],
    "pl": ["pol-Latn"],
    "pt": ["por-Latn"],
    "ro": ["ron-Latn"],
    "ru": ["rus-Cyrl"],
    "sk": ["slk-Latn"],
    "sl": ["slv-Latn"],
    "sr": ["srp-Cyrl"],
    "sv-SE": ["swe-Latn"],
    "sw": ["swa-Latn"],
    "ta": ["tam-Taml"],
    "te": ["tel-Telu"],
    "th": ["tha-Thai"],
    "tr": ["tur-Latn"],
    "uk": ["ukr-Cyrl"],
    "ur": ["urd-Arab"],
    "vi": ["vie-Latn"],
}

# Language codes for Common Voice 21
_EVAL_LANGS_CV21 = {
    "ab": ["abk-Cyrl"],
    "af": ["afr-Latn"],
    "am": ["amh-Ethi"],
    "ar": ["ara-Arab"],
    "as": ["asm-Beng"],
    "ast": ["ast-Latn"],
    "az": ["aze-Latn"],
    "ba": ["bak-Cyrl"],
    "bas": ["bas-Latn"],
    "be": ["bel-Cyrl"],
    "bg": ["bul-Cyrl"],
    "bn": ["ben-Beng"],
    "br": ["bre-Latn"],
    "ca": ["cat-Latn"],
    "ckb": ["ckb-Arab"],
    "cnh": ["cnh-Latn"],
    "cs": ["ces-Latn"],
    "cv": ["chv-Cyrl"],
    "cy": ["cym-Latn"],
    "da": ["dan-Latn"],
    "dav": ["dav-Latn"],
    "de": ["deu-Latn"],
    "dv": ["div-Thaa"],
    "dyu": ["dyu-Latn"],
    "el": ["ell-Grek"],
    "en": ["eng-Latn"],
    "eo": ["epo-Latn"],
    "es": ["spa-Latn"],
    "et": ["est-Latn"],
    "eu": ["eus-Latn"],
    "fa": ["pes-Arab"],
    "fi": ["fin-Latn"],
    "fr": ["fra-Latn"],
    "fy-NL": ["fry-Latn"],
    "ga-IE": ["gle-Latn"],
    "gl": ["glg-Latn"],
    "gn": ["grn-Latn"],
    "ha": ["hau-Latn"],
    "he": ["heb-Hebr"],
    "hi": ["hin-Deva"],
    "hsb": ["hsb-Latn"],
    "hu": ["hun-Latn"],
    "hy-AM": ["hye-Armn"],
    "ia": ["ina-Latn"],
    "id": ["ind-Latn"],
    "it": ["ita-Latn"],
    "ja": ["jpn-Jpan"],
    "ka": ["kat-Geor"],
    "kab": ["kab-Latn"],
    "kk": ["kaz-Cyrl"],
    "kln": ["kln-Latn"],
    "kmr": ["kmr-Latn"],
    "ko": ["kor-Hang"],
    "ky": ["kir-Cyrl"],
    "lg": ["lug-Latn"],
    "lij": ["lij-Latn"],
    "lt": ["lit-Latn"],
    "ltg": ["ltg-Latn"],
    "luo": ["luo-Latn"],
    "lv": ["lav-Latn"],
    "mdf": ["mdf-Cyrl"],
    "mhr": ["mhr-Cyrl"],
    "mk": ["mkd-Cyrl"],
    "ml": ["mal-Mlym"],
    "mn": ["mon-Cyrl"],
    "mr": ["mar-Deva"],
    "mrj": ["mrj-Cyrl"],
    "mt": ["mlt-Latn"],
    "myv": ["myv-Cyrl"],
    "nan-tw": ["nan-Latn"],
    "ne-NP": ["nep-Deva"],
    "nl": ["nld-Latn"],
    "nn-NO": ["nno-Latn"],
    "oc": ["oci-Latn"],
    "or": ["ori-Orya"],
    "os": ["oss-Cyrl"],
    "pa-IN": ["pan-Guru"],
    "pl": ["pol-Latn"],
    "ps": ["pus-Arab"],
    "pt": ["por-Latn"],
    "rm-sursilv": ["roh-Latn"],
    "rm-vallader": ["roh-Latn"],
    "ro": ["ron-Latn"],
    "ru": ["rus-Cyrl"],
    "rw": ["kin-Latn"],
    "sah": ["sah-Cyrl"],
    "sat": ["sat-Olck"],
    "sc": ["srd-Latn"],
    "sk": ["slk-Latn"],
    "skr": ["skr-Arab"],
    "sl": ["slv-Latn"],
    "sq": ["sqi-Latn"],
    "sr": ["srp-Cyrl"],
    "sv-SE": ["swe-Latn"],
    "sw": ["swh-Latn"],
    "ta": ["tam-Taml"],
    "te": ["tel-Telu"],
    "th": ["tha-Thai"],
    "tig": ["tig-Ethi"],
    "tk": ["tuk-Latn"],
    "tn": ["tsn-Latn"],
    "tok": ["tok-Latn"],
    "tr": ["tur-Latn"],
    "tt": ["tat-Cyrl"],
    "ug": ["uig-Arab"],
    "uk": ["ukr-Cyrl"],
    "ur": ["urd-Arab"],
    "uz": ["uzn-Latn"],
    "vi": ["vie-Latn"],
    "yi": ["yid-Hebr"],
    "yo": ["yor-Latn"],
    "yue": ["yue-Hant"],
    "zgh": ["zgh-Tfng"],
    "zh-CN": ["zho-Hans"],
    "zh-HK": ["zho-Hant"],
    "zh-TW": ["zho-Hant"],
    "zza": ["zza-Latn"],
}


class CommonVoiceMini17A2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CommonVoiceMini17A2TRetrieval",
        description="Speech recordings with corresponding text transcriptions from CommonVoice dataset.",
        reference="https://commonvoice.mozilla.org/",
        dataset={
            "path": "mteb/common_voice_17_0_mini",
            "revision": "62c6f7bb73d8cdb684868c14620ce241448e471b",
        },
        type="Any2AnyRetrieval",
        category="a2t",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS_CV17,
        main_score="hit_rate_at_5",
        date=("2020-01-01", "2024-12-31"),
        domains=["Spoken"],
        task_subtypes=["Speech Transcription Retrieval"],
        license="cc0-1.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@inproceedings{ardila2019common,
  author = {Ardila, Rosana and Branson, Megan and Davis, Kelly and Henretty, Michael and Kohler, Michael and Meyer, Josh and Morais, Reuben and Saunders, Lindsay and Tyers, Francis M and Weber, Gregor},
  booktitle = {Proceedings of the 12th Language Resources and Evaluation Conference},
  pages = {4218--4222},
  title = {Common voice: A massively-multilingual speech corpus},
  year = {2020},
}
""",
    )

    def load_data(self, **kwargs):
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.corpus = defaultdict(DatasetDict)
        self.queries = defaultdict(DatasetDict)
        self.relevant_docs = defaultdict(DatasetDict)
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self, id_col="path", text_col="sentence", audio_col="audio"):
        """Transform Common Voice dataset to MTEB a2t retrieval format.
        Process each language separately to avoid memory accumulation.
        """
        for lang in self.hf_subsets:
            for split in self.metadata.eval_splits:
                # Only load the specific split we need to save memory
                lang_dataset = datasets.load_dataset(
                    self.metadata.dataset["path"],
                    lang,
                    split=split,
                    revision=self.metadata.dataset.get("revision"),
                )

                # Create datasets directly without intermediate lists
                queries_ds = lang_dataset.select_columns(
                    [id_col, audio_col]
                ).rename_column(id_col, "id")

                corpus_ds = (
                    lang_dataset.select_columns([id_col, text_col])
                    .rename_column(id_col, "id")
                    .rename_column(text_col, "text")
                )

                # Create relevant_docs mapping
                relevant_docs_ = {
                    str(row[id_col]): {str(row[id_col]): 1} for row in lang_dataset
                }

                self.corpus[lang][split] = corpus_ds
                self.queries[lang][split] = queries_ds
                self.relevant_docs[lang][split] = relevant_docs_


class CommonVoiceMini17T2ARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CommonVoiceMini17T2ARetrieval",
        description="Speech recordings with corresponding text transcriptions from CommonVoice dataset.",
        reference="https://commonvoice.mozilla.org/",
        dataset={
            "path": "mteb/common_voice_17_0_mini",
            "revision": "62c6f7bb73d8cdb684868c14620ce241448e471b",
        },
        type="Any2AnyRetrieval",
        category="t2a",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS_CV17,
        main_score="hit_rate_at_5",
        date=("2020-01-01", "2024-12-31"),
        domains=["Spoken"],
        task_subtypes=["Speech Retrieval"],
        license="cc0-1.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@inproceedings{ardila2019common,
  author = {Ardila, Rosana and Branson, Megan and Davis, Kelly and Henretty, Michael and Kohler, Michael and Meyer, Josh and Morais, Reuben and Saunders, Lindsay and Tyers, Francis M and Weber, Gregor},
  booktitle = {Proceedings of the 12th Language Resources and Evaluation Conference},
  pages = {4218--4222},
  title = {Common voice: A massively-multilingual speech corpus},
  year = {2020},
}
""",
    )

    def load_data(self, **kwargs):
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.corpus = defaultdict(DatasetDict)
        self.queries = defaultdict(DatasetDict)
        self.relevant_docs = defaultdict(DatasetDict)
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self, id_col="path", text_col="sentence", audio_col="audio"):
        """For T2A: query=text, corpus=audio.
        Process each language separately to avoid memory accumulation.
        """
        for lang in self.hf_subsets:
            for split in self.metadata.eval_splits:
                lang_dataset = datasets.load_dataset(
                    self.metadata.dataset["path"],
                    lang,
                    split=split,
                    revision=self.metadata.dataset.get("revision"),
                )

                queries_ds = (
                    lang_dataset.select_columns([id_col, text_col])
                    .rename_column(id_col, "id")
                    .rename_column(text_col, "text")
                )

                corpus_ds = lang_dataset.select_columns(
                    [id_col, audio_col]
                ).rename_column(id_col, "id")

                relevant_docs_ = {
                    str(row[id_col]): {str(row[id_col]): 1} for row in lang_dataset
                }

                self.corpus[lang][split] = corpus_ds
                self.queries[lang][split] = queries_ds
                self.relevant_docs[lang][split] = relevant_docs_


class CommonVoiceMini21A2TRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CommonVoiceMini21A2TRetrieval",
        description="Speech recordings with corresponding text transcriptions from CommonVoice dataset.",
        reference="https://commonvoice.mozilla.org/",
        dataset={
            "path": "mteb/common_voice_21_0_mini",
            "revision": "8aef059b329d70e590bb81454a8a85ecdae54b45",
        },
        type="Any2AnyRetrieval",
        category="a2t",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS_CV21,
        main_score="hit_rate_at_5",
        date=("2020-01-01", "2024-12-31"),
        domains=["Spoken"],
        task_subtypes=["Speech Transcription Retrieval"],
        license="cc0-1.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@inproceedings{ardila2019common,
  author = {Ardila, Rosana and Branson, Megan and Davis, Kelly and Henretty, Michael and Kohler, Michael and Meyer, Josh and Morais, Reuben and Saunders, Lindsay and Tyers, Francis M and Weber, Gregor},
  booktitle = {Proceedings of the 12th Language Resources and Evaluation Conference},
  pages = {4218--4222},
  title = {Common voice: A massively-multilingual speech corpus},
  year = {2020},
}
""",
    )

    def load_data(self, **kwargs):
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.corpus = defaultdict(DatasetDict)
        self.queries = defaultdict(DatasetDict)
        self.relevant_docs = defaultdict(DatasetDict)
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self, id_col="path", text_col="sentence", audio_col="audio"):
        """Transform Common Voice dataset to MTEB a2t retrieval format.
        Process each language separately to avoid memory accumulation.
        """
        for lang in self.hf_subsets:
            for split in self.metadata.eval_splits:
                # Only load the specific split we need to save memory
                lang_dataset = datasets.load_dataset(
                    self.metadata.dataset["path"],
                    lang,
                    split=split,
                    revision=self.metadata.dataset["revision"],
                )

                # Create datasets directly without intermediate lists
                queries_ds = lang_dataset.select_columns(
                    [id_col, audio_col]
                ).rename_column(id_col, "id")

                corpus_ds = (
                    lang_dataset.select_columns([id_col, text_col])
                    .rename_column(id_col, "id")
                    .rename_column(text_col, "text")
                )

                # Create relevant_docs mapping
                relevant_docs_ = {
                    str(row[id_col]): {str(row[id_col]): 1} for row in lang_dataset
                }

                self.corpus[lang][split] = corpus_ds
                self.queries[lang][split] = queries_ds
                self.relevant_docs[lang][split] = relevant_docs_


class CommonVoiceMini21T2ARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="CommonVoiceMini21T2ARetrieval",
        description="Speech recordings with corresponding text transcriptions from CommonVoice dataset.",
        reference="https://commonvoice.mozilla.org/",
        dataset={
            "path": "mteb/common_voice_21_0_mini",
            "revision": "8aef059b329d70e590bb81454a8a85ecdae54b45",
        },
        type="Any2AnyRetrieval",
        category="t2a",
        modalities=["text", "audio"],
        eval_splits=["test"],
        eval_langs=_EVAL_LANGS_CV21,
        main_score="hit_rate_at_5",
        date=("2020-01-01", "2024-12-31"),
        domains=["Spoken"],
        task_subtypes=["Speech Retrieval"],
        license="cc0-1.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@inproceedings{ardila2019common,
  author = {Ardila, Rosana and Branson, Megan and Davis, Kelly and Henretty, Michael and Kohler, Michael and Meyer, Josh and Morais, Reuben and Saunders, Lindsay and Tyers, Francis M and Weber, Gregor},
  booktitle = {Proceedings of the 12th Language Resources and Evaluation Conference},
  pages = {4218--4222},
  title = {Common voice: A massively-multilingual speech corpus},
  year = {2020},
}
""",
    )

    def load_data(self, **kwargs):
        """Load dataset from HuggingFace hub"""
        if self.data_loaded:
            return
        self.corpus = defaultdict(DatasetDict)
        self.queries = defaultdict(DatasetDict)
        self.relevant_docs = defaultdict(DatasetDict)
        self.dataset_transform()
        self.data_loaded = True

    def dataset_transform(self, id_col="path", text_col="sentence", audio_col="audio"):
        """For T2A: query=text, corpus=audio.
        Process each language separately to avoid memory accumulation.
        """
        for lang in self.hf_subsets:
            for split in self.metadata.eval_splits:
                lang_dataset = datasets.load_dataset(
                    self.metadata.dataset["path"],
                    lang,
                    split=split,
                    revision=self.metadata.dataset["revision"],
                )

                queries_ds = (
                    lang_dataset.select_columns([id_col, text_col])
                    .rename_column(id_col, "id")
                    .rename_column(text_col, "text")
                )

                corpus_ds = lang_dataset.select_columns(
                    [id_col, audio_col]
                ).rename_column(id_col, "id")

                relevant_docs_ = {
                    str(row[id_col]): {str(row[id_col]): 1} for row in lang_dataset
                }

                self.corpus[lang][split] = corpus_ds
                self.queries[lang][split] = queries_ds
                self.relevant_docs[lang][split] = relevant_docs_



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class ArguAnaPL(AbsTaskRetrieval):
    ignore_identical_ids = True

    metadata = TaskMetadata(
        name="ArguAna-PL",
        description="ArguAna-PL",
        reference="https://huggingface.co/datasets/clarin-knext/arguana-pl",
        dataset={
            "path": "mteb/ArguAna-PL",
            "revision": "0019de73979daf75f6526a0fc5ef77ebce27b8f9",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["pol-Latn"],
        main_score="ndcg_at_10",
        date=("2023-05-01", "2024-12-31"),
        domains=["Medical", "Written"],
        task_subtypes=["Article retrieval"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated",
        bibtex_citation=r"""
@misc{wojtasik2024beirpl,
  archiveprefix = {arXiv},
  author = {Konrad Wojtasik and Vadim Shishkin and Kacper Wołowiec and Arkadiusz Janz and Maciej Piasecki},
  eprint = {2305.19840},
  primaryclass = {cs.IR},
  title = {BEIR-PL: Zero Shot Information Retrieval Benchmark for the Polish Language},
  year = {2024},
}
""",
        adapted_from=["ArguAna"],
    )



from __future__ import annotations

from typing import TYPE_CHECKING

import numpy as np

if TYPE_CHECKING:
    from numpy.typing import NDArray


def hamming_score(
    y_true: NDArray[np.integer], y_pred: NDArray[np.integer | np.floating]
) -> float:
    """Compute the Hamming score (a.k.a. label-based accuracy) for multilabel classification.

    The Hamming score is the fraction of labels that are correctly predicted for each sample,
    averaged over all samples. For samples where both y_true and y_pred have no labels,
    the score is 1.0 (perfect agreement).

    Args:
        y_true: Binary matrix of true labels with shape (n_samples, n_labels)
        y_pred: Binary matrix of predicted labels with shape (n_samples, n_labels)

    Returns:
        float: Hamming score between 0.0 and 1.0

    Raises:
        ValueError: If inputs are invalid or have incompatible shapes
        TypeError: If inputs cannot be converted to numpy arrays
    """
    y_true = np.asarray(y_true)
    y_pred = np.asarray(y_pred)

    # Check shapes
    if y_true.shape != y_pred.shape:
        raise ValueError(
            f"Shape mismatch: y_true {y_true.shape} != y_pred {y_pred.shape}"
        )

    # Check if arrays are empty
    if y_true.size == 0:
        raise ValueError("Input arrays cannot be empty")

    # Ensure 2D arrays
    if y_true.ndim != 2:
        raise ValueError(f"Arrays must be 2D, got {y_true.ndim}D")

    # Check for binary values
    if not (np.all(np.isin(y_true, [0, 1])) and np.all(np.isin(y_pred, [0, 1]))):
        raise ValueError("Arrays must contain only binary values (0 and 1)")

    # Convert to boolean for bitwise operations
    y_true_bool = y_true.astype(bool)
    y_pred_bool = y_pred.astype(bool)

    # Calculate intersection and union for each sample
    intersection = (y_true_bool & y_pred_bool).sum(axis=1)
    union = (y_true_bool | y_pred_bool).sum(axis=1)

    # Handle division by zero: when union is 0, both are all zeros, so score is 1.0
    scores = np.where(union == 0, 1.0, intersection / union)

    return float(scores.mean())



from mteb.models import sentence_transformers_loader
from mteb.models.model_meta import ModelMeta, ScoringFunction

xlmr_scandi = ModelMeta(
    loader=sentence_transformers_loader,  # type: ignore[arg-type]
    name="KFST/XLMRoberta-en-da-sv-nb",
    model_type=["dense"],
    languages=["swe-Latn", "nob-Latn", "nno-Latn", "dan-Latn", "eng-Latn"],
    open_weights=True,
    revision="d40c10ca7b1e68b5a8372f2d112dac9eb3279df1",
    release_date="2022-02-22",
    n_parameters=278043648,
    n_embedding_parameters=192_001_536,
    memory_usage_mb=1061,
    embed_dim=768,
    license="not specified",
    max_tokens=512,
    reference="https://huggingface.co/KFST/XLMRoberta-en-da-sv-nb",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["Sentence Transformers", "PyTorch", "safetensors", "Transformers"],
    use_instructions=False,
    public_training_code=None,
    public_training_data=None,
    training_datasets=None,
    adapted_from="FacebookAI/xlm-roberta-base",
)



from __future__ import annotations

import logging
import warnings
from typing import TYPE_CHECKING, Any

import numpy as np
import torch
from tqdm.auto import tqdm

from mteb._requires_package import requires_audio_dependencies, requires_package
from mteb.models import ModelMeta
from mteb.models.abs_encoder import AbsEncoder

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType
    from mteb.types._encoder_io import AudioInput


logger = logging.getLogger(__name__)


def yamnet_loader(*args, **kwargs):
    """Factory function to create a YAMNet model wrapper."""
    requires_package(
        yamnet_loader,
        "torch_vggish_yamnet",
        "google/yamnet",
        "pip install 'mteb[torch-vggish-yamnet]'",
    )
    from torch_vggish_yamnet import yamnet
    from torch_vggish_yamnet.input_proc import WaveformToInput

    class YAMNetWrapper(AbsEncoder):
        def __init__(
            self,
            device: str = "cuda" if torch.cuda.is_available() else "cpu",
            max_audio_length_seconds: float = 30.0,
            **kwargs: Any,
        ):
            requires_audio_dependencies()
            self.device = device
            self.max_audio_length_seconds = max_audio_length_seconds
            self.model = yamnet.yamnet(pretrained=True)
            self.model.eval().to(self.device)

            self.converter = WaveformToInput()
            self.sampling_rate = 16000  # YAMNet requires 16kHz audio
            self.embed_dim = 1024  # YAMNet embedding dimension
            self.min_samples = int(0.96 * self.sampling_rate)  # 15,360 samples

        def _resample_audio(self, audio, source_rate):
            """Resample audio to target sampling rate."""
            import torchaudio

            if source_rate != self.sampling_rate:
                resampler = torchaudio.transforms.Resample(
                    source_rate, self.sampling_rate
                )
                return resampler(audio)
            return audio

        def _normalize_audio(self, audio):
            """Normalize and pad audio to minimum length."""
            if isinstance(audio, np.ndarray):
                audio = torch.from_numpy(audio)

            audio = audio.float().squeeze()
            if audio.ndim > 1:
                audio = audio.mean(dim=0)

            # Apply audio truncation
            max_length = int(self.max_audio_length_seconds * self.sampling_rate)
            if audio.shape[-1] > max_length:
                audio = audio[..., :max_length]

            # Normalize to [-1.0, 1.0]
            if audio.numel() > 0 and audio.abs().max() > 1.0:
                audio = audio / audio.abs().max()

            # Pad to minimum length
            if audio.shape[-1] < self.min_samples:
                pad_amount = self.min_samples - audio.shape[-1]
                audio = torch.nn.functional.pad(audio, (0, pad_amount))

            return audio

        def _prepare_input_tensor(self, audio_data):
            """Convert audio to YAMNet input format and handle tensor dimensions."""
            if isinstance(audio_data, np.ndarray):
                audio_data = torch.from_numpy(audio_data)

            if audio_data.ndim == 1:
                audio_data = audio_data.unsqueeze(0)

            input_tensor = self.converter(audio_data.float(), self.sampling_rate).to(
                self.device
            )

            if input_tensor.numel() == 0:
                return None

            # Handle different tensor dimensions
            if input_tensor.dim() == 4 and input_tensor.shape[0] == 2:
                # [2, batch, height, width] -> [1, batch, height, width]
                input_tensor = input_tensor.mean(dim=0, keepdim=True)
            elif input_tensor.dim() == 3:
                if input_tensor.shape[0] == 3:
                    # [3, height, width] -> [1, 1, height, width]
                    input_tensor = input_tensor.mean(dim=0, keepdim=True).unsqueeze(0)
                else:
                    # [batch, height, width] -> [batch, 1, height, width]
                    input_tensor = input_tensor.unsqueeze(1)
            elif input_tensor.dim() == 2:
                # [height, width] -> [1, 1, height, width]
                input_tensor = input_tensor.unsqueeze(0).unsqueeze(0)

            return input_tensor

        def get_audio_embeddings(
            self,
            inputs: DataLoader[AudioInput],
            show_progress_bar=True,
            **kwargs,
        ):
            """Generate embeddings for audio inputs."""
            all_embeddings = []

            for batch in tqdm(
                inputs,
                desc="Processing audio",
                disable=not show_progress_bar,
            ):
                batch_embeddings = []

                for a in batch["audio"]:
                    array = torch.tensor(a["array"], dtype=torch.float32)
                    sr = a["sampling_rate"]
                    if sr is None:
                        warnings.warn(
                            f"No sampling_rate provided for an audio sample. "
                            f"Assuming {self.sampling_rate} Hz (model default)."
                        )
                        sr = self.sampling_rate

                    # Resample if needed
                    audio = self._resample_audio(array.float(), sr)
                    # Normalize
                    audio = self._normalize_audio(audio)

                    with torch.no_grad():
                        input_tensor = self._prepare_input_tensor(audio)
                        # discard logits
                        embedding, _ = self.model(input_tensor)

                        # Handle 4D tensors with shape [batch, embed_dim, 1, 1]
                        if (
                            embedding.dim() == 4
                            and embedding.shape[2] == 1
                            and embedding.shape[3] == 1
                        ):
                            # [batch, embed_dim, 1, 1] -> [batch, embed_dim]
                            embedding = embedding.squeeze(2).squeeze(2)

                        # Use mean pooling if needed
                        if len(embedding.shape) > 1:
                            embedding = torch.mean(embedding, dim=0)

                        batch_embeddings.append(embedding.cpu().detach().numpy())

                all_embeddings.extend(batch_embeddings)

            return np.array(all_embeddings)

        def encode(
            self,
            inputs: DataLoader[BatchedInput],
            *,
            task_metadata: TaskMetadata,
            hf_split: str,
            hf_subset: str,
            prompt_type: PromptType | None = None,
            **kwargs: Any,
        ) -> Array:
            if "audio" not in inputs.dataset.features:
                raise ValueError("YAMNetWrapper only supports audio inputs.")
            return self.get_audio_embeddings(inputs, **kwargs)

    return YAMNetWrapper(**kwargs)


yamnet = ModelMeta(
    loader=yamnet_loader,
    name="google/yamnet",
    languages=["eng-Latn"],
    open_weights=True,
    revision="1",
    release_date="2020-10-06",
    max_tokens=float("inf"),
    n_parameters=3_751_369,
    memory_usage_mb=14,
    embed_dim=1024,
    license="apache-2.0",
    reference="https://tfhub.dev/google/yamnet/1",
    similarity_fn_name="cosine",
    framework=["TensorFlow"],
    use_instructions=False,
    public_training_code="https://github.com/tensorflow/models/tree/master/research/audioset/yamnet",
    public_training_data="https://research.google.com/audioset/",
    training_datasets={
        "AudioSet",
    },
    modalities=["audio"],
    citation="""
@inproceedings{audioset,
  title={Audio set: An ontology and human-labeled dataset for audio events},
  author={Gemmeke, Jort F and Ellis, Daniel PW and Freedman, Dylan and Jansen, Aren and Lawrence, Wade and Moore, R Channing and Plakal, Manoj and Ritter, Marvin},
  booktitle={2017 IEEE international conference on acoustics, speech and signal processing (ICASSP)},
  pages={776--780},
  year={2017},
  organization={IEEE}
}
""",
)



from __future__ import annotations

import logging
import math
import warnings
from typing import TYPE_CHECKING, Any

import torch
from tqdm.autonotebook import tqdm

from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction
from mteb.types import PromptType

if TYPE_CHECKING:
    from PIL import Image
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput

logger = logging.getLogger(__name__)

GME_CITATION = """@misc{zhang2024gme,
      title={GME: Improving Universal Multimodal Retrieval by Multimodal LLMs},
      author={Zhang, Xin and Zhang, Yanzhao and Xie, Wen and Li, Mingxin and Dai, Ziqi and Long, Dingkun and Xie, Pengjun and Zhang, Meishan and Li, Wenjie and Zhang, Min},
      year={2024},
      eprint={2412.16855},
      archivePrefix={arXiv},
      primaryClass={cs.CL},
      url={http://arxiv.org/abs/2412.16855}
}"""


class Encoder(torch.nn.Module):
    def __init__(
        self,
        base,
        processor,
        max_length=1800,
        normalize=True,
    ) -> None:
        super().__init__()
        self.base = base
        self.processor = processor
        self.max_length = max_length
        self.normalize = normalize
        self.processor.tokenizer.padding_side = "right"
        self.default_instruction = "You are a helpful assistant."

    def forward(
        self,
        input_ids: torch.LongTensor | None = None,
        attention_mask: torch.Tensor | None = None,
        position_ids: torch.LongTensor | None = None,
        past_key_values: list[torch.FloatTensor] | None = None,
        inputs_embeds: torch.FloatTensor | None = None,
        pixel_values: torch.Tensor | None = None,
        # pixel_values_videos: torch.FloatTensor | None = None,
        image_grid_thw: torch.LongTensor | None = None,
        # video_grid_thw: torch.LongTensor | None = None,
        pooling_mask: torch.LongTensor | None = None,
        **kwargs,
    ) -> torch.Tensor:
        if inputs_embeds is None:
            inputs_embeds = self.base.model.embed_tokens(input_ids)
            if pixel_values is not None:
                pixel_values = pixel_values.type(self.base.visual.get_dtype())
                image_embeds = self.base.visual(
                    pixel_values, grid_thw=image_grid_thw
                ).to(inputs_embeds.device)
                image_mask = input_ids == self.base.config.image_token_id
                inputs_embeds[image_mask] = image_embeds
            # if pixel_values_videos is not None:
            #     pixel_values_videos = pixel_values_videos.type(self.base.visual.get_dtype())
            #     video_embeds = self.base.visual(pixel_values_videos, grid_thw=video_grid_thw).to(inputs_embeds.device)
            #     video_mask = input_ids == self.base.config.video_token_id
            #     inputs_embeds[video_mask] = video_embeds
            if attention_mask is not None:
                attention_mask = attention_mask.to(inputs_embeds.device)

        outputs = self.base.model(
            input_ids=None,
            position_ids=position_ids,
            attention_mask=attention_mask,
            past_key_values=past_key_values,
            inputs_embeds=inputs_embeds,
        )

        pooling_mask = attention_mask if pooling_mask is None else pooling_mask
        left_padding = pooling_mask[:, -1].sum() == pooling_mask.shape[0]  # TODO
        if left_padding:
            embeddings = outputs.last_hidden_state[:, -1]
        else:
            sequence_lengths = pooling_mask.sum(dim=1) - 1
            batch_size = outputs.last_hidden_state.shape[0]
            embeddings = outputs.last_hidden_state[
                torch.arange(batch_size, device=outputs.last_hidden_state.device),
                sequence_lengths,
            ]
        if self.normalize:
            embeddings = torch.nn.functional.normalize(embeddings, p=2, dim=1)
        return embeddings.contiguous()

    def embed(
        self,
        texts: list[str],
        images: list[Image.Image],
        device,
        instruction=None,
        **kwargs,
    ):
        instruction = instruction or self.default_instruction
        # Inputs must be batched
        input_texts, input_images = [], []
        for t, i in zip(texts, images):
            input_str = ""
            if i is None:
                input_images = None  # All examples in the same batch are consistent
            else:
                input_str += "<|vision_start|><|image_pad|><|vision_end|>"
                i = fetch_image(i)
                input_images.append(i)
            if t is not None:
                input_str += t
            msg = f"<|im_start|>system\n{instruction}<|im_end|>\n<|im_start|>user\n{input_str}<|im_end|>\n<|im_start|>assistant\n<|endoftext|>"
            input_texts.append(msg)

        inputs = self.processor(
            text=input_texts,
            images=input_images,
            padding=True,
            truncation=True,
            max_length=self.max_length,
            return_tensors="pt",
        )
        inputs = {k: v.to(device) for k, v in inputs.items()}  # TODO
        embeddings = self.forward(**inputs)
        return embeddings


class GmeQwen2VL(AbsEncoder):
    def __init__(
        self,
        model_name: str,
        revision: str,
        model_path: str | None = None,
        device: str = "cuda" if torch.cuda.is_available() else "cpu",
        min_image_tokens=4,
        max_image_tokens=1280,
        max_length=1800,
        **kwargs,
    ) -> None:
        from transformers import AutoModelForVision2Seq, AutoProcessor

        model_name = model_path or model_name
        base = AutoModelForVision2Seq.from_pretrained(
            model_name, revision=revision, torch_dtype=torch.float16, **kwargs
        )
        min_pixels = min_image_tokens * 28 * 28
        max_pixels = max_image_tokens * 28 * 28
        processor = AutoProcessor.from_pretrained(
            model_name,
            revision=revision,
            min_pixels=min_pixels,
            max_pixels=max_pixels,
            **kwargs,
        )
        self.model = Encoder(base, processor, max_length=max_length)
        self.model.eval()
        self.device = device
        self.sep = " "

    def encode(
        self,
        inputs: DataLoader[BatchedInput],
        *,
        task_metadata: TaskMetadata,
        hf_split: str,
        hf_subset: str,
        prompt_type: PromptType | None = None,
        show_progress_bar: bool = True,
        **kwargs: Any,
    ) -> Array:
        instruction = self.get_instruction(task_metadata, prompt_type)
        if prompt_type == PromptType.document:
            instruction = None
        elif instruction is None:
            instruction = self.get_instruction(task_metadata, prompt_type)
            # NOTE: copied from the old get_gme_instruction function.
            if isinstance(instruction, str) and instruction[-1] != ".":
                instruction += "."
        self.model = self.model.to(self.device)
        all_embeddings = []
        for batch in tqdm(inputs, disable=not show_progress_bar, desc="Fused Encoding"):
            batch_size = len(batch["text"]) or len(batch["image"])
            if "text" in batch:
                text_batch = batch["text"]
            else:
                text_batch = [None] * batch_size

            if "image" in batch:
                img_batch = batch["image"]
            else:
                img_batch = [None] * batch_size

            inputs = dict(
                texts=text_batch, images=img_batch, instruction=instruction, **kwargs
            )
            with torch.inference_mode():
                embeddings = self.model.embed(**inputs, device=self.device)
            all_embeddings.append(embeddings.cpu())
        all_embeddings = torch.cat(all_embeddings, dim=0)
        return all_embeddings


# Copied from qwen_vl_utils.vision_process.py

IMAGE_FACTOR = 28
MIN_PIXELS = 4 * 28 * 28
MAX_PIXELS = 16384 * 28 * 28
MAX_RATIO = 200


def round_by_factor(number: int, factor: int) -> int:
    """Returns the closest integer to 'number' that is divisible by 'factor'."""
    return round(number / factor) * factor


def ceil_by_factor(number: int, factor: int) -> int:
    """Returns the smallest integer greater than or equal to 'number' that is divisible by 'factor'."""
    return math.ceil(number / factor) * factor


def floor_by_factor(number: int, factor: int) -> int:
    """Returns the largest integer less than or equal to 'number' that is divisible by 'factor'."""
    return math.floor(number / factor) * factor


def smart_resize(
    height: int,
    width: int,
    factor: int = IMAGE_FACTOR,
    min_pixels: int = MIN_PIXELS,
    max_pixels: int = MAX_PIXELS,
) -> tuple[int, int]:
    """Rescales the image so that the following conditions are met:

    1. Both dimensions (height and width) are divisible by 'factor'.

    2. The total number of pixels is within the range ['min_pixels', 'max_pixels'].

    3. The aspect ratio of the image is maintained as closely as possible.
    """
    h_bar = max(factor, round_by_factor(height, factor))
    w_bar = max(factor, round_by_factor(width, factor))
    if h_bar * w_bar > max_pixels:
        beta = math.sqrt((height * width) / max_pixels)
        h_bar = floor_by_factor(height / beta, factor)
        w_bar = floor_by_factor(width / beta, factor)
    elif h_bar * w_bar < min_pixels:
        beta = math.sqrt(min_pixels / (height * width))
        h_bar = ceil_by_factor(height * beta, factor)
        w_bar = ceil_by_factor(width * beta, factor)

    if max(h_bar, w_bar) / min(h_bar, w_bar) > MAX_RATIO:
        msg = f"Absolute aspect ratio must be smaller than {MAX_RATIO}, got {max(h_bar, w_bar) / min(h_bar, w_bar)}"
        logger.warning(msg)
        warnings.warn(msg)
        if h_bar > w_bar:
            h_bar = w_bar * MAX_RATIO
        else:
            w_bar = h_bar * MAX_RATIO
    return h_bar, w_bar


def fetch_image(image: Image.Image, size_factor: int = IMAGE_FACTOR) -> Image.Image:
    from PIL import Image

    image_obj = None
    if isinstance(image, Image.Image):
        image_obj = image
    elif image.startswith("http://") or image.startswith("https://"):
        import requests

        image_obj = Image.open(requests.get(image, stream=True).raw)
    elif image.startswith("file://"):
        image_obj = Image.open(image[7:])
    elif image.startswith("data:image"):
        import base64
        from io import BytesIO

        if "base64," in image:
            _, base64_data = image.split("base64,", 1)
            data = base64.b64decode(base64_data)
            image_obj = Image.open(BytesIO(data))
    else:
        image_obj = Image.open(image)
    if image_obj is None:
        raise ValueError(
            f"Unrecognized image input, support local path, http url, base64 and PIL.Image, got {image}"
        )
    image = image_obj.convert("RGB")
    # resize
    # if "resized_height" in ele and "resized_width" in ele:
    #     resized_height, resized_width = smart_resize(
    #         ele["resized_height"],
    #         ele["resized_width"],
    #         factor=size_factor,
    #     )
    # else:
    width, height = image.size
    # min_pixels = ele.get("min_pixels", MIN_PIXELS)
    # max_pixels = ele.get("max_pixels", MAX_PIXELS)
    resized_height, resized_width = smart_resize(
        height,
        width,
        factor=size_factor,
        min_pixels=MIN_PIXELS,
        max_pixels=MAX_PIXELS,
    )
    image = image.resize((resized_width, resized_height))

    return image


training_data = {
    "MSMARCO",
    "NQ",
    "NQHardNegatives",
    "NanoNQRetrieval",
    "NQ-PL",  # translation not trained on
    "HotpotQA",
    "HotpotQA-PL",  # translation not trained on
    "HotpotQAHardNegatives",
    # TriviaQA (Joshi et al., 2017),
    # SQuAD (Rajpurkar et al., 2016),
    "FEVER",
    # AllNLI for SimCSE (Gao et al., 2021), selecting a total of 1 million entries.
    # ImageNet (Deng et al., 2009)
    # LAION (Schuhmann et al., 2022),
    # mscoco (Lin et al., 2014),
    # Docmatix (Laurenc¸on et al., 2024)
    # synthetic data
    # M-BEIR (Wei et al., 2024)
}


gme_qwen2vl_2b = ModelMeta(
    loader=GmeQwen2VL,
    name="Alibaba-NLP/gme-Qwen2-VL-2B-Instruct",
    model_type=["dense"],
    languages=["eng-Latn", "cmn-Hans"],
    open_weights=True,
    revision="ce765ae71b8cdb208203cd8fb64a170b1b84293a",
    release_date="2024-12-24",
    modalities=["image", "text"],
    n_parameters=2208985600,
    n_embedding_parameters=233_373_696,
    memory_usage_mb=8427,
    embed_dim=1536,
    license="apache-2.0",
    max_tokens=32768,
    reference="https://huggingface.co/Alibaba-NLP/gme-Qwen2-VL-2B-Instruct",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=training_data,
    citation=GME_CITATION,
)

gme_qwen2vl_7b = ModelMeta(
    loader=GmeQwen2VL,
    name="Alibaba-NLP/gme-Qwen2-VL-7B-Instruct",
    model_type=["dense"],
    languages=["eng-Latn", "cmn-Hans"],
    open_weights=True,
    revision="477027a6480f8630363be77751f169cc3434b673",
    release_date="2024-12-24",
    modalities=["image", "text"],
    n_parameters=7746378240,
    n_embedding_parameters=544_997_376,
    memory_usage_mb=31629,
    embed_dim=3584,
    license="apache-2.0",
    max_tokens=32768,
    reference="https://huggingface.co/Alibaba-NLP/gme-Qwen2-VL-7B-Instruct",
    similarity_fn_name=ScoringFunction.COSINE,
    framework=["PyTorch", "Sentence Transformers", "safetensors", "Transformers"],
    use_instructions=True,
    public_training_code=None,
    public_training_data=None,
    training_datasets=training_data,
    citation=GME_CITATION,
)



from __future__ import annotations

from typing import TYPE_CHECKING, Any, Literal

import torch
from tqdm.auto import tqdm

from mteb._requires_package import requires_image_dependencies
from mteb.models.abs_encoder import AbsEncoder
from mteb.models.model_meta import ModelMeta, ScoringFunction

if TYPE_CHECKING:
    from torch.utils.data import DataLoader

    from mteb.abstasks.task_metadata import TaskMetadata
    from mteb.types import Array, BatchedInput, PromptType

VISTA_CITATION = """@article{zhou2024vista,
  title={VISTA: Visualized Text Embedding For Universal Multi-Modal Retrieval},
  author={Zhou, Junjie and Liu, Zheng and Xiao, Shitao and Zhao, Bo and Xiong, Yongping},
  journal={arXiv preprint arXiv:2406.04292},
  year={2024}
}"""


def vista_loader(model_name, **kwargs):
    try:  # a temporal fix for the dependency issues of vista models.
        from visual_bge.modeling import Visualized_BGE
    except ImportError:
        raise ImportError(
            "Please install `visual_bge`, refer to https://github.com/FlagOpen/FlagEmbedding/tree/master/research/visual_bge#install-flagembedding."
        )

    class VisualizedBGEWrapper(Visualized_BGE, AbsEncoder):
        """Setting up VISTA

        ```
        git clone https://github.com/FlagOpen/FlagEmbedding.git
        cd FlagEmbedding/research/visual_bge
        pip install -e .
        pip install torchvision timm einops ftfy
        ```
        back to the root folder of mteb; download the vision tower for bge-base
        ```
        cd ..
        wget https://huggingface.co/BAAI/bge-visualized/resolve/main/Visualized_base_en_v1.5.pth?download=true
        ```
        rename it to `visualized_base_en_V1.5.pth`
        ```
        mv Visualized_base_en_v1.5.pth?download=true visualized_base_en_V1.5.pth
        ```
        download the vision tower for bge-m3
        ```
        wget https://huggingface.co/BAAI/bge-visualized/resolve/main/Visualized_m3.pth?download=true
        ```
        rename it to `visualized_m3.pth`
        ```
        mv Visualized_m3.pth?download=true visualized_m3.pth
        ```
        """

        def __init__(
            self,
            model_name_bge: str | None = None,
            model_weight=None,
            normlized: bool = True,
            sentence_pooling_method: str = "cls",
            negatives_cross_device: bool = False,
            temperature: float = 0.02,
            from_pretrained=None,
            image_tokens_num: int | None = None,
            **kwargs: Any,
        ):
            requires_image_dependencies()

            super().__init__(
                model_name_bge=model_name_bge,
                model_weight=model_weight,
                normlized=normlized,
                sentence_pooling_method=sentence_pooling_method,
                negatives_cross_device=negatives_cross_device,
                temperature=temperature,
                from_pretrained=from_pretrained,
            )
            self.image_tokens_num = image_tokens_num
            self.max_text_len_with_image = (
                self.tokenizer.model_max_length - image_tokens_num
            )
            self.eval()

        def encode_text(self, texts: dict[str, torch.Tensor]) -> Array:
            """Currently override Visualized_BGE's the original implementation
            to fix attention_mask & embedding_output dtype misalignment

            Args:
                texts: {"input_ids": ..., "attention_mask": ...}

            Returns:
                Array of text embeddings
            """
            input_ids = texts["input_ids"]
            attention_mask = texts["attention_mask"]

            input_shape = input_ids.size()
            device = input_ids.device

            token_type_ids = torch.zeros(input_shape, dtype=torch.long, device=device)

            head_mask = [None] * self.depth
            extended_attention_mask: torch.Tensor = self.get_extended_attention_mask(
                attention_mask, input_shape
            )

            embedding_output = self.bge_embeddings(
                input_ids=input_ids,
                position_ids=None,
                token_type_ids=token_type_ids,
                inputs_embeds=None,
                past_key_values_length=0,
            )

            # this line is missing in vista, currently override "encode_text" only to fix this.
            extended_attention_mask = extended_attention_mask.to(embedding_output.dtype)

            encoder_outputs = self.bge_encoder(
                embedding_output,
                attention_mask=extended_attention_mask,
                head_mask=head_mask,
                encoder_hidden_states=None,
                encoder_attention_mask=None,
                past_key_values=None,
                use_cache=False,
                output_attentions=False,
                output_hidden_states=False,
                return_dict=True,
            )
            sequence_output = encoder_outputs[0]

            t_reps = self.sentence_embedding(
                sequence_output, texts["attention_mask"]
            )  # tensor: reps with pooling
            if self.normlized:
                t_reps = torch.nn.functional.normalize(t_reps, dim=-1)
            return t_reps.contiguous()

        def get_text_embeddings(
            self,
            texts: DataLoader[BatchedInput],
            show_progress_bar: bool = True,
            prompt_type: PromptType | None = None,
            input_type: Literal["document", "query"] | None = None,
            **kwargs: Any,
        ):
            all_text_embeddings = []
            for batch in tqdm(
                texts, disable=not show_progress_bar, desc="Text Encoding"
            ):
                with torch.no_grad():
                    texts = self.tokenizer(
                        batch["text"],
                        return_tensors="pt",
                        padding=True,
                        truncation=True,
                    )
                    batch_embeddings = self.encode_text(texts.to(self.device))
                all_text_embeddings.append(batch_embeddings.cpu())
            return torch.cat(all_text_embeddings, dim=0)

        def get_image_embeddings(
            self,
            images: DataLoader[BatchedInput],
            show_progress_bar: bool = True,
            prompt_type: PromptType | None = None,
            input_type: Literal["document", "query"] | None = None,
            **kwargs: Any,
        ):
            all_image_embeddings = []
            with torch.no_grad():
                for batch in tqdm(images):
                    imgs = [self.preprocess_val(image) for image in batch["image"]]
                    imgs = torch.stack(imgs)

                    batch_embeddings = self.encode_image(images=imgs.to(self.device))
                    all_image_embeddings.append(batch_embeddings.cpu())
            return torch.cat(all_image_embeddings, dim=0)

        def encode(
            self,
            inputs: DataLoader[BatchedInput],
            *,
            task_metadata: TaskMetadata,
            hf_split: str,
            hf_subset: str,
            prompt_type: PromptType | None = None,
            show_progress_bar: bool = True,
            **kwargs: Any,
        ) -> Array:
            if "text" in inputs.dataset.features and "image" in inputs.dataset.features:
                all_fused_embeddings = []
                with torch.no_grad():
                    for batch in tqdm(
                        inputs,
                        disable=not show_progress_bar,
                        desc="Interleaved Encoding",
                    ):
                        texts = self.tokenizer(
                            batch["text"],
                            return_tensors="pt",
                            padding=True,
                            truncation=True,
                            max_length=self.max_text_len_with_image,
                        )
                        images = [
                            self.preprocess_val(image) for image in batch["image"]
                        ]
                        images = torch.stack(images)
                        all_fused_embeddings.append(
                            self.encode_mm(
                                images.to(self.device), texts.to(self.device)
                            )
                            .cpu()
                            .to(torch.float32)
                        )
                return torch.cat(all_fused_embeddings, dim=0)
            elif "text" in inputs.dataset.features:
                return self.get_text_embeddings(
                    inputs,
                    task_metadata=task_metadata,
                    prompt_type=prompt_type,
                    **kwargs,
                )
            elif "image" in inputs.dataset.features:
                return self.get_image_embeddings(
                    inputs,
                    task_metadata=task_metadata,
                    prompt_type=prompt_type,
                    **kwargs,
                )
            raise ValueError

    return VisualizedBGEWrapper(model_name, **kwargs)


vista_training_datasets = set(
    # VISTA_S2
)

visualized_bge_base = ModelMeta(
    loader=vista_loader,
    loader_kwargs=dict(
        model_weight="visualized_base_en_V1.5.pth",
        image_tokens_num=196,
    ),
    name="BAAI/bge-visualized-base",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="98db10b10d22620010d06f11733346e1c98c34aa",
    release_date="2024-06-06",
    modalities=["image", "text"],
    n_parameters=196_000_000,
    n_embedding_parameters=None,
    memory_usage_mb=1631,
    max_tokens=512,
    embed_dim=768,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/JUNJIE99/VISTA_S2",
    framework=["PyTorch"],
    reference="https://huggingface.co/BAAI/bge-visualized",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=vista_training_datasets,
    citation=VISTA_CITATION,
)

visualized_bge_m3 = ModelMeta(
    loader=vista_loader,
    loader_kwargs=dict(
        model_weight="visualized_m3.pth",
        image_tokens_num=256,
    ),
    name="BAAI/bge-visualized-m3",
    model_type=["dense"],
    languages=["eng-Latn"],
    revision="98db10b10d22620010d06f11733346e1c98c34aa",
    release_date="2024-06-06",
    modalities=["image", "text"],
    n_parameters=872_909_505,
    n_embedding_parameters=None,
    memory_usage_mb=4263,
    max_tokens=8192,
    embed_dim=1024,
    license=None,
    open_weights=True,
    public_training_code=None,
    public_training_data="https://huggingface.co/datasets/JUNJIE99/VISTA_S2",
    framework=["PyTorch"],
    reference="https://huggingface.co/BAAI/bge-visualized",
    similarity_fn_name=ScoringFunction.COSINE,
    use_instructions=False,
    training_datasets=vista_training_datasets,
    citation=VISTA_CITATION,
)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class VisRAGRetInfoVQA(AbsTaskRetrieval):
    """VisRAG Retrieval task for InfoVQA infographics.

    The corpus contains infographic images and the queries are questions about
    those infographics.  Each query points to one relevant infographic image.
    """

    metadata = TaskMetadata(
        name="VisRAGRetInfoVQA",
        description="Evaluate the retrieval and understanding of complex infographics where layout and graphical elements are essential for cross-modal question answering.",
        reference="https://arxiv.org/abs/2104.12756",
        type="Retrieval",
        task_subtypes=["Image Text Retrieval"],
        category="t2i",
        modalities=["text", "image"],
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="recall_at_10",
        dataset={
            "path": "mteb/VisRAGRetInfoVQA",
            "revision": "5890a9860a153222b97e641abccb8bcc237722c1",
        },
        date=("2010-01-01", "2020-12-31"),
        domains=["Web"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation="""@misc{mathew2021infographicvqa,
  archiveprefix = {arXiv},
  author = {Minesh Mathew and Viraj Bagal and Rubèn Pérez Tito and Dimosthenis Karatzas and Ernest Valveny and C. V Jawahar},
  eprint = {2104.12756},
  primaryclass = {cs.CV},
  title = {InfographicVQA},
  url = {https://arxiv.org/abs/2104.12756},
  year = {2021},
}""",
    )



from .swedish_patent_cpc_group_classification import SwedishPatentCPCGroupClassification
from .swedish_patent_cpc_subclass_classification import (
    SwedishPatentCPCSubclassClassification,
)

__all__ = [
    "SwedishPatentCPCGroupClassification",
    "SwedishPatentCPCSubclassClassification",
]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class BillSumCARetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        dataset={
            "path": "isaacus/mteb-BillSumCA",
            "revision": "5014f29d7fdde6f9073a75b72be53ed73eed60c6",
        },
        name="BillSumCA",
        description="A benchmark for retrieving Californian bills based on their summaries.",
        reference="https://huggingface.co/datasets/FiscalNote/billsum",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2024-08-14", "2025-07-18"),
        domains=["Legal", "Government"],
        task_subtypes=[],
        license="cc0-1.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{Eidelman_2019,
  author = {Eidelman, Vladimir},
  booktitle = {Proceedings of the 2nd Workshop on New Frontiers in Summarization},
  doi = {10.18653/v1/d19-5406},
  pages = {48–56},
  publisher = {Association for Computational Linguistics},
  title = {BillSum: A Corpus for Automatic Summarization of US Legislation},
  url = {http://dx.doi.org/10.18653/v1/D19-5406},
  year = {2019},
}
""",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class IEMOCAPEmotionClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="IEMOCAPEmotion",
        description="Classification of speech samples into emotions (angry, happy, sad, neutral, frustrated, excited, fearful, surprised, disgusted) from interactive emotional dyadic conversations.",
        reference="https://doi.org/10.1007/s10579-008-9076-6",
        dataset={
            "path": "mteb/iemocap",
            "revision": "6d4225271da423e791e16770d335cfa351cdf88e",
        },
        type="AudioClassification",
        category="a2c",
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=("2008-01-01", "2008-12-31"),
        domains=["Spoken", "Speech"],
        task_subtypes=["Emotion classification"],
        license="cc-by-nc-sa-3.0",
        annotations_creators="expert-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="created",
        bibtex_citation=r"""
@article{busso2008iemocap,
  author = {Busso, Carlos and Bulut, Murtaza and Lee, Chi-Chun and Kazemzadeh, Abe and Mower, Emily and Kim, Samuel and Chang, Jeannette N and Lee, Sungbok and Narayanan, Shrikanth S},
  journal = {Language resources and evaluation},
  number = {4},
  pages = {335--359},
  publisher = {Springer},
  title = {IEMOCAP: Interactive emotional dyadic motion capture database},
  volume = {42},
  year = {2008},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "emotion"

    is_cross_validation: bool = True

    def dataset_transform(self):
        # Define emotion labels and their mapping to indices
        labels = [
            "angry",  # 0
            "sad",  # 1
            "happy",  # 2
            "neutral",  # 3
            "frustrated",  # 4
            "excited",  # 5
            "fear",  # 6
            "surprise",  # 7
            "disgust",  # 8
            "other",  # 9
        ]
        label2id = {emotion: idx for idx, emotion in enumerate(labels)}

        # Basic filtering to ensure we have valid emotion labels
        for split in self.dataset:
            # First ensure we have valid emotion labels and normalize case
            self.dataset[split] = self.dataset[split].filter(
                lambda example: example["major_emotion"] is not None
                and example["major_emotion"] != ""
            )

            # Map to indices with case normalization for reliability
            self.dataset[split] = self.dataset[split].map(
                lambda example: {
                    "emotion_id": label2id.get(example["major_emotion"].lower(), -1)
                }
            )

            # Filter out any examples with unknown emotions
            self.dataset[split] = self.dataset[split].filter(
                lambda example: example["emotion_id"] != -1
            )

            # Use numeric ID as the label
            self.dataset[split] = self.dataset[split].rename_column(
                "emotion_id", self.label_column_name
            )



from .urdu_roman_sentiment_classification import (
    UrduRomanSentimentClassification,
    UrduRomanSentimentClassificationV2,
)

__all__ = ["UrduRomanSentimentClassification", "UrduRomanSentimentClassificationV2"]



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata

DOMAINS = ["writing", "recreation", "science", "technology", "lifestyle"]
DOMAINS_TYPES = ["search", "forum"]
HF_SUBSETS = [f"{d}_{t}" for d in DOMAINS for t in DOMAINS_TYPES]


class LoTTERetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="LoTTE",
        dataset={
            "path": "mteb/LoTTE",
            "revision": "a887e9f8e3d24427fdff98c8bba3198a6853a929",
        },
        description=(
            "LoTTE (Long-Tail Topic-stratified Evaluation for IR) is designed to evaluate retrieval models "
            "on underrepresented, long-tail topics. Unlike MSMARCO or BEIR, LoTTE features domain-specific queries and "
            "passages from StackExchange (covering writing, recreation, science, technology, and lifestyle), providing "
            "a challenging out-of-domain generalization benchmark."
        ),
        type="Retrieval",
        modalities=["text"],
        category="t2t",
        reference="https://github.com/stanford-futuredata/ColBERT/blob/main/LoTTE.md",
        eval_splits=["test", "dev"],
        eval_langs={domain: ["eng-Latn"] for domain in HF_SUBSETS},
        # in paper mentioned as `Success@5 (sometimes referred to as Recall@5)`,
        # which is hit_rate@5 in code https://github.com/stanford-futuredata/ColBERT/blob/cc4f3dc91c0b45d2d08c251d9d95178285c65f1c/utility/evaluate/evaluate_lotte_rankings.py#L39-L40
        main_score="hit_rate_at_5",
        date=("2021-12-02", "2022-06-10"),
        domains=["Academic", "Web", "Social"],
        task_subtypes=["Article retrieval"],
        license="mit",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{santhanam-etal-2022-colbertv2,
  address = {Seattle, United States},
  author = {Santhanam, Keshav  and
Khattab, Omar  and
Saad-Falcon, Jon  and
Potts, Christopher  and
Zaharia, Matei},
  booktitle = {Proceedings of the 2022 Conference of the North American Chapter of the Association for Computational Linguistics: Human Language Technologies},
  doi = {10.18653/v1/2022.naacl-main.272},
  editor = {Carpuat, Marine  and
de Marneffe, Marie-Catherine  and
Meza Ruiz, Ivan Vladimir},
  month = jul,
  pages = {3715--3734},
  publisher = {Association for Computational Linguistics},
  title = {{C}ol{BERT}v2: Effective and Efficient Retrieval via Lightweight Late Interaction},
  url = {https://aclanthology.org/2022.naacl-main.272/},
  year = {2022},
}
""",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class GunshotTriangulation(AbsTaskClassification):
    metadata = TaskMetadata(
        name="GunshotTriangulation",
        description="Classifying a weapon based on its muzzle blast",
        reference="https://huggingface.co/datasets/anime-sh/GunshotTriangulationHEAR",
        dataset={
            "path": "mteb/GunshotTriangulationHear",
            "revision": "d5caa98a10a41dd8890a343a631fdbaacc747108",
        },
        type="AudioClassification",
        category="a2c",
        eval_splits=["train"],
        eval_langs=["zxx-Zxxx"],
        main_score="accuracy",
        date=("2025-03-07", "2025-03-07"),
        domains=[],  # Replace with appropriate domain from allowed list?? No appropriate domain name is available
        task_subtypes=["Gunshot Audio Classification"],
        license="not specified",  # Replace with appropriate license from allowed list
        annotations_creators="derived",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@misc{raponi2021soundgunsdigitalforensics,
  archiveprefix = {arXiv},
  author = {Simone Raponi and Isra Ali and Gabriele Oligeri},
  eprint = {2004.07948},
  primaryclass = {eess.AS},
  title = {Sound of Guns: Digital Forensics of Gun Audio Samples meets Artificial Intelligence},
  url = {https://arxiv.org/abs/2004.07948},
  year = {2021},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "label"

    is_cross_validation: bool = True



from __future__ import annotations

from typing import TYPE_CHECKING, Protocol

if TYPE_CHECKING:
    from collections.abc import Callable

    from mteb.types import Array, TopRankedDocumentsType


class IndexEncoderSearchProtocol(Protocol):
    """Protocol for search backends used in encoder-based retrieval."""

    def add_documents(
        self,
        embeddings: Array,
        idxs: list[str],
    ) -> None:
        """Add documents to the search backend.

        Args:
            embeddings: Embeddings of the documents to add.
            idxs: IDs of the documents to add.
        """

    def search(
        self,
        embeddings: Array,
        top_k: int,
        similarity_fn: Callable[[Array, Array], Array],
        top_ranked: TopRankedDocumentsType | None = None,
        query_idx_to_id: dict[int, str] | None = None,
    ) -> tuple[list[list[float]], list[list[int]]]:
        """Search through added corpus embeddings or rerank top-ranked documents.

        Supports both full-corpus and reranking search modes:
            - Full-corpus mode: `top_ranked=None`, uses added corpus embeddings.
            - Reranking mode:  `top_ranked` contains mapping {query_id: [doc_ids]}.

        Args:
            embeddings: Query embeddings, shape (num_queries, dim).
            top_k: Number of top results to return.
            similarity_fn: Function to compute similarity between query and corpus.
            top_ranked: Mapping of query_id -> list of candidate doc_ids. Used for reranking.
            query_idx_to_id: Mapping of query index -> query_id. Used for reranking.

        Returns:
            A tuple (top_k_values, top_k_indices), for each query:
                - top_k_values: List of top-k similarity scores.
                - top_k_indices: List of indices of the top-k documents in the added corpus.
        """

    def clear(self) -> None:
        """Clear all stored documents and embeddings from the backend."""



from .ambient_acoustic_context import AmbientAcousticContextClassification
from .beijing_opera import BeijingOpera
from .bird_clef import BirdCLEFClassification
from .esc50 import ESC50Classification
from .gtzan_genre import GTZANGenre
from .gunshot_triangulation import GunshotTriangulation
from .mridingham_stroke import MridinghamStroke
from .mridingham_tonic import MridinghamTonic
from .n_synth import NSynth
from .tau_acoustic_scenes_2022_mobile import TAUAcousticScenes2022Mobile
from .tut_acoustic_scenes import TUTAcousticScenesClassification
from .urban_sound8k import UrbanSound8kClassification

__all__ = [
    "AmbientAcousticContextClassification",
    "BeijingOpera",
    "BirdCLEFClassification",
    "ESC50Classification",
    "GTZANGenre",
    "GunshotTriangulation",
    "MridinghamStroke",
    "MridinghamTonic",
    "NSynth",
    "TAUAcousticScenes2022Mobile",
    "TUTAcousticScenesClassification",
    "UrbanSound8kClassification",
]



from .ger_da_lir_retrieval import GerDaLIR
from .ger_da_lir_small_retrieval import GerDaLIRSmall
from .german1_retrieval import German1Retrieval
from .german_dpr_retrieval import GermanDPR
from .german_gov_service_retrieval import GermanGovServiceRetrieval
from .german_healthcare1_retrieval import GermanHealthcare1Retrieval
from .german_legal1_retrieval import GermanLegal1Retrieval
from .german_qu_ad_retrieval import GermanQuADRetrieval
from .legal_qu_ad_retrieval import LegalQuAD

__all__ = [
    "GerDaLIR",
    "GerDaLIRSmall",
    "German1Retrieval",
    "GermanDPR",
    "GermanGovServiceRetrieval",
    "GermanHealthcare1Retrieval",
    "GermanLegal1Retrieval",
    "GermanQuADRetrieval",
    "LegalQuAD",
]



from mteb.abstasks.multilabel_classification import (
    AbsTaskMultilabelClassification,
)
from mteb.abstasks.task_metadata import TaskMetadata


class CEDRClassification(AbsTaskMultilabelClassification):
    metadata = TaskMetadata(
        name="CEDRClassification",
        dataset={
            "path": "mteb/CEDRClassification",
            "revision": "d872ceae33d049f0ef50e8abb9229b569ebfcf7d",
        },
        description="Classification of sentences by emotions, labeled into 5 categories (joy, sadness, surprise, fear, and anger).",
        reference="https://www.sciencedirect.com/science/article/pii/S1877050921013247",
        type="MultilabelClassification",
        category="t2c",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["rus-Cyrl"],
        main_score="accuracy",
        date=("1999-01-01", "2021-09-01"),
        domains=["Web", "Social", "Blog", "Written"],
        task_subtypes=["Sentiment/Hate speech"],
        license="apache-2.0",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@article{sboev2021data,
  author = {Sboev, Alexander and Naumov, Aleksandr and Rybka, Roman},
  journal = {Procedia Computer Science},
  pages = {637--642},
  publisher = {Elsevier},
  title = {Data-Driven Model for Emotion Detection in Russian Texts},
  volume = {190},
  year = {2021},
}
""",
        prompt="Given a comment as query, find expressed emotions (joy, sadness, surprise, fear, and anger)",
    )



from mteb._evaluators.retrieval_metrics import robustness_at_10
from mteb.abstasks import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class InstructIR(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="InstructIR",
        description='A benchmark specifically designed to evaluate the instruction following ability in information retrieval models. Our approach focuses on user-aligned instructions tailored to each query instance, reflecting the diverse characteristics inherent in real-world search scenarios. **NOTE**: scores on this may differ unless you include instruction first, then "[SEP]" and then the query via redefining `combine_query_and_instruction` in your model.',
        reference="https://github.com/kaistAI/InstructIR/tree/main",
        dataset={
            "path": "mteb/InstructIR-mteb",
            "revision": "42c3afabe480643b755a7099dbf0f9ebeedaf6ca",
        },
        type="InstructionRetrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="robustness_at_10",
        date=("2024-02-05", "2024-02-06"),
        domains=["Web"],
        task_subtypes=["Article retrieval"],
        license="mit",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@article{oh2024instructir,
  archiveprefix = {{arXiv}},
  author = {{Hanseok Oh and Hyunji Lee and Seonghyeon Ye and Haebin Shin and Hansol Jang and Changwook Jun and Minjoon Seo}},
  eprint = {{2402.14334}},
  primaryclass = {{cs.CL}},
  title = {{INSTRUCTIR: A Benchmark for Instruction Following of Information Retrieval Models}},
  year = {{2024}},
}
""",
    )

    def task_specific_scores(
        self,
        scores: dict[str, dict[str, float]],
        qrels: dict[str, dict[str, int]],
        results: dict[str, dict[str, float]],
        hf_split: str,
        hf_subset: str,
    ) -> dict[str, float]:
        return {"robustness_at_10": robustness_at_10(qrels, results, scores)}



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class STSBenchmarkSTS(AbsTaskSTS):
    min_score = 0
    max_score = 5

    metadata = TaskMetadata(
        name="STSBenchmark",
        dataset={
            "path": "mteb/stsbenchmark-sts",
            "revision": "b0fddb56ed78048fa8b90373c8a3cfc37b684831",
        },
        description="Semantic Textual Similarity Benchmark (STSbenchmark) dataset.",
        reference="https://github.com/PhilipMay/stsb-multi-mt/",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="cosine_spearman",
        date=("2021-01-01", "2021-12-31"),  # publication year
        domains=["Blog", "News", "Written"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="machine-translated and verified",
        bibtex_citation=r"""
@inproceedings{huggingface:dataset:stsb_multi_mt,
  author = {Philip May},
  title = {Machine translated multilingual STS benchmark dataset.},
  url = {https://github.com/PhilipMay/stsb-multi-mt},
  year = {2021},
}
""",
    )

    min_score = 0
    max_score = 5



"""Test cases for the _load_from_cache and _rebuild_from_full_repository methods."""

from unittest.mock import MagicMock, patch

import pytest

from mteb.cache import ResultCache
from mteb.results import BenchmarkResults


class TestLoadFromCache:
    """Test the _load_from_cache method."""

    def test_rebuild_flag_forces_full_rebuild(self, tmp_path):
        """Test rebuild=True bypasses cache and forces rebuild."""
        cache = ResultCache(cache_path=tmp_path)
        cache_filename = "test_cache.json"
        expected_path = tmp_path / "leaderboard" / cache_filename
        expected_path.parent.mkdir(parents=True, exist_ok=True)
        expected_path.write_text('{"test": "should be ignored"}')

        with patch.object(cache, "_rebuild_from_full_repository") as mock_rebuild:
            mock_result = MagicMock(spec=BenchmarkResults)
            mock_rebuild.return_value = mock_result
            result = cache._load_from_cache(cache_filename, rebuild=True)
            mock_rebuild.assert_called_once_with(expected_path)
            assert result == mock_result

    def test_loading_strategies_in_order(self, tmp_path):
        """Test the 3-tier loading strategy: local -> download -> rebuild."""
        cache = ResultCache(cache_path=tmp_path)
        cache_filename = "test_cache.json"
        expected_path = tmp_path / "leaderboard" / cache_filename
        expected_path.parent.mkdir(parents=True, exist_ok=True)
        mock_result = MagicMock(spec=BenchmarkResults)

        # Test 1: Load from existing local cache
        expected_path.write_text("{}")
        with patch("mteb.results.BenchmarkResults.from_disk") as mock_from_disk:
            mock_from_disk.return_value = mock_result
            result = cache._load_from_cache(cache_filename, rebuild=False)
            mock_from_disk.assert_called_once_with(expected_path)
            assert result == mock_result

        # Test 2: Download from cached-data branch when local doesn't exist
        expected_path.unlink()  # Remove local file
        with (
            patch.object(cache, "_download_cached_results_from_branch") as mock_dl,
            patch("mteb.results.BenchmarkResults.from_disk") as mock_from_disk,
        ):
            mock_dl.return_value = expected_path
            mock_from_disk.return_value = mock_result
            result = cache._load_from_cache(cache_filename, rebuild=False)
            mock_dl.assert_called_once_with(output_path=expected_path)
            assert result == mock_result

        # Test 3: Fallback to full rebuild when download fails
        with (
            patch.object(cache, "_download_cached_results_from_branch") as mock_dl,
            patch.object(cache, "_rebuild_from_full_repository") as mock_rebuild,
        ):
            mock_dl.side_effect = Exception("Download failed")
            mock_rebuild.return_value = mock_result
            result = cache._load_from_cache(cache_filename, rebuild=False)
            mock_rebuild.assert_called_once_with(expected_path)
            assert result == mock_result

    def test_corrupt_cache_triggers_fallback(self, tmp_path):
        """Test that corrupt cache files trigger next strategy."""
        cache = ResultCache(cache_path=tmp_path)
        cache_filename = "test_cache.json"
        expected_path = tmp_path / "leaderboard" / cache_filename
        expected_path.parent.mkdir(parents=True, exist_ok=True)
        expected_path.write_text("invalid json {{{")

        with (
            patch("mteb.results.BenchmarkResults.from_disk") as mock_from_disk,
            patch.object(cache, "_download_cached_results_from_branch") as mock_dl,
            patch.object(cache, "_rebuild_from_full_repository") as mock_rebuild,
        ):
            mock_from_disk.side_effect = Exception("Invalid JSON")
            mock_dl.side_effect = Exception("Download failed")
            mock_result = MagicMock(spec=BenchmarkResults)
            mock_rebuild.return_value = mock_result
            result = cache._load_from_cache(cache_filename, rebuild=False)
            mock_rebuild.assert_called_once_with(expected_path)
            assert result == mock_result


class TestRebuildFromFullRepository:
    """Test the _rebuild_from_full_repository method."""

    def test_full_rebuild_process(self, tmp_path):
        """Test rebuild downloads repo, loads results, and saves cache."""
        cache = ResultCache(cache_path=tmp_path)
        quick_cache_path = tmp_path / "cache.json"

        with (
            patch.object(cache, "download_from_remote") as mock_download,
            patch.object(cache, "load_results") as mock_load_results,
            patch("mteb.get_model_metas") as mock_get_model_metas,
        ):
            # Mock model metas - None names should be filtered
            meta1 = MagicMock()
            meta1.name = "model1"
            meta2 = MagicMock()
            meta2.name = "model2"
            meta3 = MagicMock()
            meta3.name = None  # Should be filtered
            mock_get_model_metas.return_value = [meta1, meta2, meta3]
            mock_results = MagicMock(spec=BenchmarkResults)
            mock_load_results.return_value = mock_results

            result = cache._rebuild_from_full_repository(quick_cache_path)

            mock_download.assert_called_once()
            mock_load_results.assert_called_once_with(
                models=[meta1.name, meta2.name],
                only_main_score=True,
                require_model_meta=False,
                include_remote=True,
            )
            mock_results.to_disk.assert_called_once_with(quick_cache_path)
            assert result == mock_results

    def test_rebuild_error_propagation(self, tmp_path):
        """Test that errors during rebuild are properly propagated."""
        cache = ResultCache(cache_path=tmp_path)
        quick_cache_path = tmp_path / "cache.json"

        # Test download failure
        with patch.object(cache, "download_from_remote") as mock_download:
            mock_download.side_effect = Exception("Network error")
            with pytest.raises(Exception, match="Network error"):
                cache._rebuild_from_full_repository(quick_cache_path)

        # Test load_results failure
        with (
            patch.object(cache, "download_from_remote"),
            patch.object(cache, "load_results") as mock_load_results,
            patch("mteb.get_model_metas") as mock_get_model_metas,
        ):
            meta = MagicMock()
            meta.name = "model1"
            mock_get_model_metas.return_value = [meta]
            mock_load_results.side_effect = Exception("Load failed")
            with pytest.raises(Exception, match="Load failed"):
                cache._rebuild_from_full_repository(quick_cache_path)



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class SCIDOCS(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="SCIDOCS",
        dataset={
            "path": "mteb/scidocs",
            "revision": "f8c2fcf00f625baaa80f62ec5bd9e1fff3b8ae88",
        },
        description=(
            "SciDocs, a new evaluation benchmark consisting of seven document-level tasks ranging from citation"
            + " prediction, to document classification and recommendation."
        ),
        reference="https://allenai.org/data/scidocs",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=("2000-01-01", "2020-12-31"),
        domains=["Academic", "Written", "Non-fiction"],
        task_subtypes=["Scientific Reranking"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{specter2020cohan,
  author = {Arman Cohan and Sergey Feldman and Iz Beltagy and Doug Downey and Daniel S. Weld},
  booktitle = {ACL},
  title = {SPECTER: Document-level Representation Learning using Citation-informed Transformers},
  year = {2020},
}
""",
        prompt={
            "query": "Given a scientific paper title, retrieve paper abstracts that are cited by the given paper"
        },
    )



from mteb.abstasks import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata


class BiossesSTSVN(AbsTaskSTS):
    metadata = TaskMetadata(
        name="BIOSSES-VN",
        dataset={
            "path": "GreenNode/biosses-sts-vn",
            "revision": "1dae4a6df91c0852680cd4ab48c8c1d8a9ed49b2",
        },
        description="A translated dataset from Biomedical Semantic Similarity Estimation. The process of creating the VN-MTEB (Vietnamese Massive Text Embedding Benchmark) from English samples involves a new automated system: - The system uses large language models (LLMs), specifically Coherence's Aya model, for translation. - Applies advanced embedding models to filter the translations. - Use LLM-as-a-judge to scoring the quality of the samples base on multiple criteria.",
        reference="https://tabilab.cmpe.boun.edu.tr/BIOSSES/DataSet.html",
        type="STS",
        category="t2c",
        eval_splits=["test"],
        eval_langs=["vie-Latn"],
        main_score="cosine_spearman",
        date=("2025-07-29", "2025-07-30"),
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated and LM verified",
        domains=["Medical"],
        task_subtypes=[],
        bibtex_citation=r"""
@misc{pham2025vnmtebvietnamesemassivetext,
  archiveprefix = {arXiv},
  author = {Loc Pham and Tung Luu and Thu Vo and Minh Nguyen and Viet Hoang},
  eprint = {2507.21500},
  primaryclass = {cs.CL},
  title = {VN-MTEB: Vietnamese Massive Text Embedding Benchmark},
  url = {https://arxiv.org/abs/2507.21500},
  year = {2025},
}
""",
        adapted_from=["BIOSSES"],
    )
    min_score = 0
    max_score = 5



from .esc50 import ESC50ZeroshotClassification
from .urban_sound8k import UrbanSound8kZeroshotClassification

__all__ = [
    "ESC50ZeroshotClassification",
    "UrbanSound8kZeroshotClassification",
]



from mteb.abstasks import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class BuiltBenchReranking(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="BuiltBenchReranking",
        description="Reranking of built asset entity type/class descriptions given a query describing an entity as represented in well-established industry classification systems such as Uniclass, IFC, etc.",
        reference="https://arxiv.org/abs/2411.12056",
        dataset={
            "path": "mteb/BuiltBenchReranking",
            "revision": "63343e96ffc63a82470fc5bf919ad13951df6c15",
        },
        type="Reranking",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="map_at_1000",
        date=("2024-06-01", "2024-11-30"),
        domains=["Engineering", "Written"],
        task_subtypes=[],
        license="cc-by-nd-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="created",
        bibtex_citation=r"""
@article{shahinmoghadam2024benchmarking,
  author = {Shahinmoghadam, Mehrzad and Motamedi, Ali},
  journal = {arXiv preprint arXiv:2411.12056},
  title = {Benchmarking pre-trained text embedding models in aligning built asset information},
  year = {2024},
}
""",
        prompt={
            "query": "Given a query, retrieve relevant entity descriptions from buit asset classification systems such as IFC and Uniclass"
        },
    )



from mteb.abstasks.task_metadata import TaskMetadata
from mteb.abstasks.text.bitext_mining import AbsTaskBitextMining


class DanishMedicinesAgencyBitextMining(AbsTaskBitextMining):
    metadata = TaskMetadata(
        name="DanishMedicinesAgencyBitextMining",
        dataset={
            "path": "mteb/english-danish-parallel-corpus",
            "revision": "0e45e58e9c360134cdb2bf023ad2606a27a2a086",
        },
        description="A Bilingual English-Danish parallel corpus from The Danish Medicines Agency.",
        reference="https://sprogteknologi.dk/dataset/bilingual-english-danish-parallel-corpus-from-the-danish-medicines-agency",
        type="BitextMining",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["dan-Latn", "eng-Latn"],
        main_score="f1",
        date=("2016-01-01", "2019-02-24"),
        domains=["Medical", "Written"],
        task_subtypes=[],
        license="https://opendefinition.org/od/2.1/en/",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{elrc_danish_medicines_agency_2018,
  author = {Rozis, Roberts},
  institution = {European Union},
  license = {Open Under-PSI},
  note = {Dataset created within the European Language Resource Coordination (ELRC) project under the Connecting Europe Facility - Automated Translation (CEF.AT) actions SMART 2014/1074 and SMART 2015/1091.},
  title = {Bilingual English-Danish Parallel Corpus from the Danish Medicines Agency},
  url = {https://sprogteknologi.dk/dataset/bilingual-english-danish-parallel-corpus-from-the-danish-medicines-agency},
  year = {2019},
}
""",
    )



from mteb.abstasks.classification import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class AmbientAcousticContextClassification(AbsTaskClassification):
    metadata = TaskMetadata(
        name="AmbientAcousticContext",
        description="The Ambient Acoustic Context dataset contains 1-second segments for activities that occur in a workplace setting. This is a downsampled version with ~100 train and ~50 test samples per class.",
        reference="https://dl.acm.org/doi/10.1145/3379503.3403535",
        dataset={
            "path": "mteb/ambient-acoustic-context-small",
            "revision": "8f4de158d4162de768ebb4dc0594429d785077da",
        },
        type="AudioClassification",
        category="a2c",
        eval_splits=["test"],
        eval_langs=["zxx-Zxxx"],
        main_score="accuracy",
        date=("2020-01-01", "2020-12-31"),  # Paper publication date
        domains=["Spoken", "Speech"],
        task_subtypes=["Environment Sound Classification"],
        license="not specified",  # Not specified in dataset card
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{10.1145/3379503.3403535,
  address = {New York, NY, USA},
  articleno = {33},
  author = {Park, Chunjong and Min, Chulhong and Bhattacharya, Sourav and Kawsar, Fahim},
  booktitle = {22nd International Conference on Human-Computer Interaction with Mobile Devices and Services},
  doi = {10.1145/3379503.3403535},
  isbn = {9781450375160},
  keywords = {Acoustic ambient context, Conversational agents},
  location = {Oldenburg, Germany},
  numpages = {9},
  publisher = {Association for Computing Machinery},
  series = {MobileHCI '20},
  title = {Augmenting Conversational Agents with Ambient Acoustic Contexts},
  url = {https://doi.org/10.1145/3379503.3403535},
  year = {2020},
}
""",
    )

    input_column_name: str = "audio"
    label_column_name: str = "label"

    is_cross_validation: bool = False



from collections import defaultdict

from datasets import load_dataset

from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class NanoFEVERRetrieval(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="NanoFEVERRetrieval",
        description="NanoFEVER is a smaller version of "
        + "FEVER (Fact Extraction and VERification), which consists of 185,445 claims generated by altering sentences"
        + " extracted from Wikipedia and subsequently verified without knowledge of the sentence they were"
        + " derived from.",
        reference="https://fever.ai/",
        dataset={
            "path": "zeta-alpha-ai/NanoFEVER",
            "revision": "a8bfdf1bf15181167a7e22e69cf8754bdea9b4c8",
        },
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["train"],
        eval_langs=["eng-Latn"],
        main_score="ndcg_at_10",
        date=["2018-01-01", "2018-12-31"],
        domains=["Academic", "Encyclopaedic"],
        task_subtypes=["Claim verification"],
        license="cc-by-4.0",
        annotations_creators="expert-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{thorne-etal-2018-fever,
  address = {New Orleans, Louisiana},
  author = {Thorne, James  and
Vlachos, Andreas  and
Christodoulopoulos, Christos  and
Mittal, Arpit},
  booktitle = {Proceedings of the 2018 Conference of the North {A}merican Chapter of the Association for Computational Linguistics: Human Language Technologies, Volume 1 (Long Papers)},
  doi = {10.18653/v1/N18-1074},
  editor = {Walker, Marilyn  and
Ji, Heng  and
Stent, Amanda},
  month = jun,
  pages = {809--819},
  publisher = {Association for Computational Linguistics},
  title = {{FEVER}: a Large-scale Dataset for Fact Extraction and {VER}ification},
  url = {https://aclanthology.org/N18-1074},
  year = {2018},
}
""",
        prompt={
            "query": "Given a claim, retrieve documents that support or refute the claim"
        },
        adapted_from=["FEVER"],
    )

    def load_data(self, num_proc: int | None = None, **kwargs) -> None:
        if self.data_loaded:
            return

        self.corpus = load_dataset(
            "zeta-alpha-ai/NanoFEVER",
            "corpus",
            revision="a8bfdf1bf15181167a7e22e69cf8754bdea9b4c8",
        )
        self.queries = load_dataset(
            "zeta-alpha-ai/NanoFEVER",
            "queries",
            revision="a8bfdf1bf15181167a7e22e69cf8754bdea9b4c8",
        )
        self.relevant_docs = load_dataset(
            "zeta-alpha-ai/NanoFEVER",
            "qrels",
            revision="a8bfdf1bf15181167a7e22e69cf8754bdea9b4c8",
        )

        self.corpus = {
            split: {
                sample["_id"]: {"_id": sample["_id"], "text": sample["text"]}
                for sample in self.corpus[split]
            }
            for split in self.corpus
        }

        self.queries = {
            split: {sample["_id"]: sample["text"] for sample in self.queries[split]}
            for split in self.queries
        }

        relevant_docs = {}

        for split in self.relevant_docs:
            relevant_docs[split] = defaultdict(dict)
            for query_id, corpus_id in zip(
                self.relevant_docs[split]["query-id"],
                self.relevant_docs[split]["corpus-id"],
            ):
                relevant_docs[split][query_id][corpus_id] = 1
        self.relevant_docs = relevant_docs

        self.data_loaded = True



from mteb.abstasks import AbsTaskClassification
from mteb.abstasks.task_metadata import TaskMetadata


class RESISC45Classification(AbsTaskClassification):
    input_column_name: str = "image"
    samples_per_label: int = 16
    n_experiments: int = 5

    metadata = TaskMetadata(
        name="RESISC45",
        description="Remote Sensing Image Scene Classification by Northwestern Polytechnical University (NWPU).",
        reference="https://ieeexplore.ieee.org/abstract/document/7891544",
        dataset={
            "path": "mteb/resisc45",
            "revision": "a5b7c0d47eb6544fb2ef0d380b9c105f3c6dc259",
        },
        type="ImageClassification",
        category="i2c",
        eval_splits=["test"],
        eval_langs=["eng-Latn"],
        main_score="accuracy",
        date=(
            "2017-01-01",
            "2017-03-01",
        ),  # Estimated range for the collection of reviews
        domains=["Encyclopaedic"],
        task_subtypes=["Object recognition"],
        license="not specified",
        annotations_creators="derived",
        dialect=[],
        modalities=["image"],
        sample_creation="created",
        bibtex_citation=r"""
@article{7891544,
  author = {Cheng, Gong and Han, Junwei and Lu, Xiaoqiang},
  doi = {10.1109/JPROC.2017.2675998},
  journal = {Proceedings of the IEEE},
  keywords = {Remote sensing;Benchmark testing;Spatial resolution;Social network services;Satellites;Image analysis;Machine learning;Unsupervised learning;Classification;Benchmark data set;deep learning;handcrafted features;remote sensing image;scene classification;unsupervised feature learning},
  number = {10},
  pages = {1865-1883},
  title = {Remote Sensing Image Scene Classification: Benchmark and State of the Art},
  volume = {105},
  year = {2017},
}
""",
    )



from mteb.abstasks.sts import AbsTaskSTS
from mteb.abstasks.task_metadata import TaskMetadata

_LANGUAGES = {
    "en": ["eng-Latn"],
    "de": ["deu-Latn"],
    "es": ["spa-Latn"],
    "pl": ["pol-Latn"],
    "tr": ["tur-Latn"],
    "ar": ["ara-Arab"],
    "ru": ["rus-Cyrl"],
    "zh": ["cmn-Hans"],
    "fr": ["fra-Latn"],
    "de-en": ["deu-Latn", "eng-Latn"],
    "es-en": ["spa-Latn", "eng-Latn"],
    "it": ["ita-Latn"],
    "pl-en": ["pol-Latn", "eng-Latn"],
    "zh-en": ["cmn-Hans", "eng-Latn"],
    "es-it": ["spa-Latn", "ita-Latn"],
    "de-fr": ["deu-Latn", "fra-Latn"],
    "de-pl": ["deu-Latn", "pol-Latn"],
    "fr-pl": ["fra-Latn", "pol-Latn"],
}


class STS22CrosslingualSTSv2(AbsTaskSTS):
    fast_loading = True
    metadata = TaskMetadata(
        name="STS22.v2",
        dataset={
            "path": "mteb/sts22-crosslingual-sts",
            "revision": "d31f33a128469b20e357535c39b82fb3c3f6f2bd",
        },
        description="SemEval 2022 Task 8: Multilingual News Article Similarity. Version 2 filters updated on STS22 by removing pairs where one of entries contain empty sentences.",
        reference="https://competitions.codalab.org/competitions/33835",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="cosine_spearman",
        date=("2020-01-01", "2020-06-11"),
        domains=["News", "Written"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{chen-etal-2022-semeval,
  address = {Seattle, United States},
  author = {Chen, Xi  and
Zeynali, Ali  and
Camargo, Chico  and
Fl{\"o}ck, Fabian  and
Gaffney, Devin  and
Grabowicz, Przemyslaw  and
Hale, Scott  and
Jurgens, David  and
Samory, Mattia},
  booktitle = {Proceedings of the 16th International Workshop on Semantic Evaluation (SemEval-2022)},
  doi = {10.18653/v1/2022.semeval-1.155},
  editor = {Emerson, Guy  and
Schluter, Natalie  and
Stanovsky, Gabriel  and
Kumar, Ritesh  and
Palmer, Alexis  and
Schneider, Nathan  and
Singh, Siddharth  and
Ratan, Shyam},
  month = jul,
  pages = {1094--1106},
  publisher = {Association for Computational Linguistics},
  title = {{S}em{E}val-2022 Task 8: Multilingual news article similarity},
  url = {https://aclanthology.org/2022.semeval-1.155},
  year = {2022},
}
""",
        adapted_from=["STS22"],
    )

    min_score = 1
    max_score = 4


class STS22CrosslingualSTS(AbsTaskSTS):
    fast_loading = True
    metadata = TaskMetadata(
        name="STS22",
        dataset={
            "path": "mteb/sts22-crosslingual-sts",
            "revision": "de9d86b3b84231dc21f76c7b7af1f28e2f57f6e3",
        },
        description="SemEval 2022 Task 8: Multilingual News Article Similarity",
        reference="https://competitions.codalab.org/competitions/33835",
        type="STS",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=_LANGUAGES,
        main_score="cosine_spearman",
        date=("2020-01-01", "2020-06-11"),
        domains=["News", "Written"],
        task_subtypes=[],
        license="not specified",
        annotations_creators="human-annotated",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@inproceedings{chen-etal-2022-semeval,
  address = {Seattle, United States},
  author = {Chen, Xi  and
Zeynali, Ali  and
Camargo, Chico  and
Fl{\"o}ck, Fabian  and
Gaffney, Devin  and
Grabowicz, Przemyslaw  and
Hale, Scott  and
Jurgens, David  and
Samory, Mattia},
  booktitle = {Proceedings of the 16th International Workshop on Semantic Evaluation (SemEval-2022)},
  doi = {10.18653/v1/2022.semeval-1.155},
  editor = {Emerson, Guy  and
Schluter, Natalie  and
Stanovsky, Gabriel  and
Kumar, Ritesh  and
Palmer, Alexis  and
Schneider, Nathan  and
Singh, Siddharth  and
Ratan, Shyam},
  month = jul,
  pages = {1094--1106},
  publisher = {Association for Computational Linguistics},
  title = {{S}em{E}val-2022 Task 8: Multilingual news article similarity},
  url = {https://aclanthology.org/2022.semeval-1.155},
  year = {2022},
}
""",
        superseded_by="STS22.v2",
    )

    min_score = 1
    max_score = 4



from mteb.abstasks.pair_classification import AbsTaskPairClassification
from mteb.abstasks.task_metadata import TaskMetadata


class ArmenianParaphrasePC(AbsTaskPairClassification):
    metadata = TaskMetadata(
        name="ArmenianParaphrasePC",
        description="asparius/Armenian-Paraphrase-PC",
        reference="https://github.com/ivannikov-lab/arpa-paraphrase-corpus",
        dataset={
            "path": "mteb/ArmenianParaphrasePC",
            "revision": "594f25dfef459cbfa30fc1765a6fb2053244f85b",
        },
        type="PairClassification",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["hye-Armn"],
        main_score="max_ap",
        date=("2021-01-01", "2022-04-06"),
        domains=["News", "Written"],
        task_subtypes=[],
        license="apache-2.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="found",
        bibtex_citation=r"""
@misc{malajyan2020arpa,
  archiveprefix = {arXiv},
  author = {Arthur Malajyan and Karen Avetisyan and Tsolak Ghukasyan},
  eprint = {2009.12615},
  primaryclass = {cs.CL},
  title = {ARPA: Armenian Paraphrase Detection Corpus and Models},
  year = {2020},
}
""",
    )



from datasets import Sequence, Value
from sklearn.linear_model import LogisticRegression
from sklearn.multioutput import MultiOutputClassifier

from mteb.abstasks import AbsTaskMultilabelClassification
from mteb.abstasks.task_metadata import TaskMetadata


class BirdSetMultilabelClassification(AbsTaskMultilabelClassification):
    metadata = TaskMetadata(
        name="BirdSet",
        description="BirdSet: A Large-Scale Dataset for Audio Classification in Avian Bioacoustics",
        reference="https://huggingface.co/datasets/DBD-research-group/BirdSet",
        dataset={
            "path": "mteb/BirdSet",
            "name": "HSN",
            "revision": "bdaa5020a8dc594a9a1d3b344e6ca9dbfaa33c74",
        },
        type="AudioMultilabelClassification",
        category="a2t",
        eval_splits=["test_5s"],
        eval_langs=["zxx-Zxxx"],
        main_score="accuracy",
        date=("2025-01-01", "2025-12-31"),  # Competition year
        domains=["Spoken", "Speech", "Bioacoustics"],
        task_subtypes=["Species Classification"],
        license="cc-by-nc-4.0",
        annotations_creators="human-annotated",
        dialect=[],
        modalities=["audio"],
        sample_creation="created",
        bibtex_citation=r"""
@misc{rauch2024birdsetlargescaledatasetaudio,
  archiveprefix = {arXiv},
  author = {Lukas Rauch and Raphael Schwinger and Moritz Wirth and René Heinrich and Denis Huseljic and Marek Herde and Jonas Lange and Stefan Kahl and Bernhard Sick and Sven Tomforde and Christoph Scholz},
  eprint = {2403.10380},
  primaryclass = {cs.SD},
  title = {BirdSet: A Large-Scale Dataset for Audio Classification in Avian Bioacoustics},
  url = {https://arxiv.org/abs/2403.10380},
  year = {2024},
}
""",
    )

    evaluator_model = MultiOutputClassifier(estimator=LogisticRegression())
    input_column_name: str = "audio"
    label_column_name: str = "labels"
    samples_per_label: int = 21

    def dataset_transform(self):
        """Rename ebird_code_multilabel → labels and turn IDs → bird names."""
        if "ebird_code_multilabel" in self.dataset.column_names[self.eval_splits[0]]:
            self.dataset = self.dataset.rename_column(
                original_column_name="ebird_code_multilabel",
                new_column_name=self.label_column_name,  # "labels"
            )

        id2name = {
            0: "Gray-crowned Rosy-Finch",
            1: "White-crowned Sparrow",
            2: "American Pipit",
            3: "Spotted Sandpiper",
            4: "Rock Wren",
            5: "Brewer's Blackbird",
            6: "Dark-eyed Junco",
            7: "Fox Sparrow",
            8: "Clark's Nutcracker",
            9: "Mountain Bluebird",
            10: "Cassin's Finch",
            11: "Mallard",
            12: "Hermit Thrush",
            13: "American Robin",
            14: "Yellow-rumped Warbler",
            15: "Yellow Warbler",
            16: "Dusky Flycatcher",
            17: "Mountain Chickadee",
            18: "Orange-crowned Warbler",
            19: "Warbling Vireo",
            20: "Northern Flicker",
        }

        def ids_to_names(example):
            """Convert a list of ints to a list of bird-name strings."""
            labels = example[self.label_column_name]
            if labels is None:
                return {self.label_column_name: []}
            return {self.label_column_name: [id2name[i] for i in labels]}

        # Cast to int64 to remove ClassLabel schema before mapping to strings
        self.dataset = self.dataset.cast_column(
            self.label_column_name, Sequence(Value("int64"))
        )

        self.dataset = self.dataset.map(
            ids_to_names,
            num_proc=4,
        )

        # Filter out empty labels
        self.dataset = self.dataset.filter(
            lambda x: x[self.label_column_name] is not None
            and len(x[self.label_column_name]) > 0
        )

        # Only subsample splits that are larger than n_samples to avoid division by zero
        n_samples = 2048
        splits_to_subsample = [
            split for split in self.eval_splits if len(self.dataset[split]) > n_samples
        ]

        if splits_to_subsample:
            self.dataset = self.stratified_subsampling(
                self.dataset,
                seed=self.seed,
                splits=splits_to_subsample,
                label=self.label_column_name,
                n_samples=n_samples,
            )



from .abstask import AbsTask
from .classification import AbsTaskClassification
from .clustering import AbsTaskClustering
from .clustering_legacy import AbsTaskClusteringLegacy
from .image.image_text_pair_classification import AbsTaskImageTextPairClassification
from .multilabel_classification import AbsTaskMultilabelClassification
from .pair_classification import AbsTaskPairClassification
from .regression import AbsTaskRegression
from .retrieval import AbsTaskRetrieval
from .sts import AbsTaskSTS
from .text.bitext_mining import AbsTaskBitextMining
from .text.reranking import AbsTaskReranking
from .text.summarization import AbsTaskSummarization
from .zeroshot_classification import AbsTaskZeroShotClassification

__all__ = [
    "AbsTask",
    "AbsTaskBitextMining",
    "AbsTaskClassification",
    "AbsTaskClustering",
    "AbsTaskClusteringLegacy",
    "AbsTaskImageTextPairClassification",
    "AbsTaskMultilabelClassification",
    "AbsTaskPairClassification",
    "AbsTaskRegression",
    "AbsTaskReranking",
    "AbsTaskRetrieval",
    "AbsTaskSTS",
    "AbsTaskSummarization",
    "AbsTaskZeroShotClassification",
]



"""Simplified version of https://gist.github.com/AlexeyVatolin/ea3adc21aa7a767603ff393b22085adc from https://github.com/embeddings-benchmark/mteb/pull/2900"""

from __future__ import annotations

import logging
from typing import TYPE_CHECKING

import datasets
import pandas as pd
from datasets import DatasetDict

if TYPE_CHECKING:
    from datasets import Dataset

    from mteb import TaskMetadata

logger = logging.getLogger(__name__)


def deduplicate(dataset: Dataset, input_column: str) -> Dataset:
    """Remove duplicate texts, keeping the first occurrence."""
    unique_texts = set()
    indices_to_keep = []
    for i, text in enumerate(dataset[input_column]):
        text = text.strip()
        if text not in unique_texts:
            unique_texts.add(text)
            indices_to_keep.append(i)

    logger.info(
        f"[deduplicate] removed={len(dataset) - len(indices_to_keep)}/{len(dataset)}"
    )
    return dataset.select(indices_to_keep)


def filter_empty(dataset: Dataset, input_column: str) -> Dataset:
    """Filter out empty or whitespace-only examples."""
    before = len(dataset)
    ds = dataset.filter(lambda x: len(x[input_column].strip()) > 0)
    logger.info(f"[filter_empty] removed={before - len(ds)}/{before}")
    return ds


def filter_train_leakage(
    train_dataset: Dataset, test_dataset: Dataset, input_column: str
) -> Dataset:
    """Remove test examples that appear in training."""
    train_texts = set(train_dataset[input_column])
    before = len(test_dataset)
    indices = [
        i
        for i, text in enumerate(test_dataset[input_column])
        if text not in train_texts
    ]
    logger.info(f"[filter_train_leakage] removed={before - len(indices)}/{before}")
    return test_dataset.select(indices)


def filter_unclear_label(
    dataset_dict: DatasetDict, input_column: str, label_column: str
) -> DatasetDict:
    """Remove examples where the same text appears with multiple different labels."""
    normalized: dict[str, set[str | tuple[str, ...]]] = {}
    logger.debug("[filter_controversial] scanning dataset for label conflicts...")

    for split, ds in dataset_dict.items():
        for text, label in zip(ds[input_column], ds[label_column]):
            key = text.strip().lower()
            normalized.setdefault(key, set()).add(
                label if isinstance(label, (str, int, float)) else tuple(label)  # type: ignore[arg-type]
            )

    bad_texts = {t for t, labels in normalized.items() if len(labels) > 1}
    logger.info(f"[filter_controversial] Removing {len(bad_texts)} conflicting texts")

    new_dict = {}
    for split, ds in dataset_dict.items():
        before = len(ds)
        filtered = ds.filter(lambda x: x[input_column].strip().lower() not in bad_texts)
        logger.debug(
            f"[filter_controversial:{split}] removed={before - len(filtered)}/{before}"
        )
        new_dict[split] = filtered

    return DatasetDict(new_dict)


def filter_short(dataset: Dataset, input_column: str, min_words: int = 3) -> Dataset:
    """Filter out texts with fewer than `min_words`."""
    before = len(dataset)
    ds = dataset.filter(lambda x: len(x[input_column].strip().split()) >= min_words)
    logger.debug(f"[filter_short] removed={before - len(ds)}/{before}")
    return ds


def split_train_test(
    ds: DatasetDict,
    metadata: TaskMetadata,
    train_split: str,
    label_column: str,
) -> DatasetDict:
    if train_split in ds and metadata.eval_splits == train_split:
        before = len(ds[train_split])
        logger.info(
            f"[split_train_test] eval_splits == train_split; performing split on {before} examples"
        )
        ds[train_split] = ds[train_split].cast_column(
            label_column,
            datasets.ClassLabel(names=list(set(ds[train_split][label_column]))),
        )
        label_counts = pd.Series(ds[train_split][label_column]).value_counts()
        one_sample_labels = set(label_counts[label_counts == 1].index.tolist())

        if one_sample_labels:
            logger.info(
                f"[split_train_test] Removing {len(one_sample_labels)} labels with only one instance"
            )
            ds[train_split] = ds[train_split].filter(
                lambda x: x[label_column] not in one_sample_labels
            )

        splits = ds[train_split].train_test_split(
            test_size=min(2048, before // 2), seed=42, stratify_by_column=label_column
        )
        ds = DatasetDict({train_split: splits[train_split], "test": splits["test"]})
        metadata.eval_splits = ["test"]
        logger.info(
            f"[split_train_test] Train size={len(ds[train_split])}, Test size={len(ds['test'])}"
        )

    return ds



from mteb.abstasks.retrieval import AbsTaskRetrieval
from mteb.abstasks.task_metadata import TaskMetadata


class HotpotQANL(AbsTaskRetrieval):
    metadata = TaskMetadata(
        name="HotpotQA-NL",
        dataset={
            "path": "clips/beir-nl-hotpotqa",
            "revision": "3ab9eaff3e1d6f7f81914ae3b2bdd1e8511cf9a8",
        },
        description=(
            "HotpotQA is a question answering dataset featuring natural, multi-hop questions, with strong"
            + "supervision for supporting facts to enable more explainable question answering systems. HotpotQA-NL is "
            "a Dutch translation. "
        ),
        reference="https://hotpotqa.github.io/",
        type="Retrieval",
        category="t2t",
        modalities=["text"],
        eval_splits=["test"],
        eval_langs=["nld-Latn"],
        main_score="ndcg_at_10",
        date=("2018-01-01", "2018-12-31"),  # best guess: based on publication date
        domains=["Web", "Written"],
        task_subtypes=["Question answering"],
        license="cc-by-sa-4.0",
        annotations_creators="derived",
        dialect=[],
        sample_creation="machine-translated and verified",  # manually checked a small subset
        bibtex_citation=r"""
@misc{banar2024beirnlzeroshotinformationretrieval,
  archiveprefix = {arXiv},
  author = {Nikolay Banar and Ehsan Lotfi and Walter Daelemans},
  eprint = {2412.08329},
  primaryclass = {cs.CL},
  title = {BEIR-NL: Zero-shot Information Retrieval Benchmark for the Dutch Language},
  url = {https://arxiv.org/abs/2412.08329},
  year = {2024},
}
""",
        adapted_from=["HotpotQA"],
    )
